@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 2000-2011 Seiji Kaneko. All rights reserved.
+** Copyright (c) 2000-2014 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     convert.h	input side converter common header
-    $Id: convert.h,v 1.108 2013/07/22 13:45:12 seiji Exp seiji $
+    $Id: convert.h,v 1.111 2015/04/26 15:08:38 seiji Exp seiji $
 */
 #ifndef CONVERT_H_DEFINED
 #define CONVERT_H_DEFINED
@@ -63,7 +63,7 @@
 #define is_upper(x)	((x >= 'A') && (x <= 'Z'))
 #define is_cname_char(x) ((x == '-') || ((x >= '0') && (x <= '9')) \
 			|| ((x >= 'A') && (x <= 'Z')) || (x == '_') \
-			|| ((x >= 'a') && (x <= 'z')) || (x == '.'))
+			|| ((x >= 'a') && (x <= 'z')))
 #define is_ascii(x)	((x >= 0x21) && (x < 0x7f))
 
 #define SKFtolower(x)	(((x>='A')&&(x<='Z'))? (x+0x20) : x)
@@ -84,9 +84,7 @@ extern int	be_in P_((skfFILE *));
 extern int	esc_process P_((skfFILE *));
 extern int	c1_process P_((skfFILE *, int));
 extern int	seq_sweep P_((skfFILE *, int));
-#ifdef	FUTURESUPPORT
 extern int	is_idnspace P_((skf_ucode));
-#endif
 
 extern int	cns11643_defined_code;
 
@@ -154,9 +152,9 @@ extern struct iso_byte_defs iso_iso8859_defs[];
 extern struct iso_byte_defs iso_3_dblbyte_defs[];
 extern struct iso_byte_defs iso_4_dblbyte_defs[];
 extern struct iso_byte_defs misc_byte_defs[];
-extern struct iso_byte_defs miscmul_byte_defs[];
 extern struct iso_byte_defs cp_byte_defs[];
 extern struct iso_byte_defs ovlay_byte_defs[];
+extern struct iso_byte_defs priv_dblbyte_defs[];
 
 /* --------------------------------------------------------------- */
 /* is_kana flags 						   */
@@ -177,6 +175,7 @@ extern struct iso_byte_defs ovlay_byte_defs[];
 #define COD_25		0x00020000UL	/* GR/GL double set	   */
 #define COD_MB		0x00002000UL	/* Multibyte		   */
 #define COD_MB_4	0x00040000UL	/* ISO MB 4octet code	   */
+#define COD_PRIV	0x00080000UL	/* ISO private area	   */
 #define COD_X213_1	0x00000020UL	/* jis x-0213 1st plane	   */
 #define COD_X213_2	0x00000010UL	/* jis x-0213 2nd plane	   */
 #define COD_X213_MSK	0x0000203fUL	/* jis x-0213 		   */
@@ -226,8 +225,6 @@ extern struct iso_byte_defs ovlay_byte_defs[];
 #define JOHAB_HANJA1	0xe0
 #define JOHAB_HANJA1E	0xf9
 
-#define	is_multibyte(x) (((x & 0x01) == 0) && (x != 0))
-
 /* --------------------------------------------------------------- */
 /* input codetable predefinitions				   */
 /* --------------------------------------------------------------- */
@@ -458,7 +455,7 @@ void shift_cond_recovery();
 /* various utilities elsewhere 					   */
 /* --------------------------------------------------------------- */
 extern void	keis_conv P_((int, int));
-extern int	jef_conv P_((int, int));
+extern void	jef_conv P_((int, int));
 extern void	is_sbroken P_((int, int));
 
 extern int	x0201conv P_((int, int));
@@ -588,24 +585,19 @@ extern char	*skf_table_lmagic;
 /* Unic*de decomposition/composition table definitions		   */
 /* --------------------------------------------------------------- */
 /* #define UNI_NKDL_TBL_LEN 13376 :* 0x00a0-0x33ff,0xf900-0xffff   */
-#define UNI_NKDL_TBL_LEN 20480	/* 0x00a0-0x33ff,0xf900-0xffff     */
+#define UNI_NKDL_TBL_LEN 17280	/* 0x00a0-0x33ff,0xf900-0xffff     */
 				/* 0x1d100-0x1d7ff,0x2f800-0x2fa20 */
 
-#define UNI_NORM_TBL_LEN 20480	/* (Unicode 5.0 = 16849) * 1.1	   */
+#define UNI_NORM_TBL_LEN 18600	/* (Unicode 5.0 = 16849) * 1.1	   */
 #define UNI_LAT_OFF	160	/* 0x00a0			   */
 #define UNI_UP_OFF	13152	/* 0x33ff - 0x00a0		   */
 #define UNI_COM_OFF	14944	/* UP_OFF + (0xf800 - 0xffff)	   */
 #define UNI_CJKC_OFF	16736	/* COM_OFF + (0x1d7ff - 0x1d100)   */
-#define UNI_KAITHI_OFF	17280	/* CJKC_OFF + (0x2f800 - 0x2fa1f)  */
-#define UNI_ARAM_OFF	18672	/* KAITHI_OFF + (0x11600 - 0x11090) */
-#define UNI_ECSP_OFF	18928	/* ARAM_OFF + (0x1ef00 - 0x1ee00)  */
-#define UNI_TAIL_OFF	19696	/* ECSP_OFF + (0x1f400 - 0x1f100)  */
 #define UNI_ATTRIB_TBL_LEN 65536
 
 extern unsigned short	*nkdc_lowptr;	/* area mapper		   */
 extern skf_ucode	*nkdc_lowmap;	/* map char table (in BSP) */
 extern unsigned short	*nkd_strmap;	/* map for compose strength*/
-extern int		nkdc_lowsize;	/* map char table length   */
 
 #define UNIHAN_SBASE	0xac00	/* Hangul start point		   */
 #define UNIHAN_LBASE	0x1100
@@ -663,7 +655,6 @@ extern int skf_rot13conv_d P_((int));
 #define punycode_unexpeof	-7
 
 extern skf_ucode puny_adapt P_ ((long, long, int));
-extern int	is_prohibit P_ ((skf_ucode));
 extern int	in_ace;
 #endif
 
