@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 1997-2012 Seiji Kaneko. All rights reserved.
+** Copyright (c) 1997-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
 ** error.c:	error message related routine in skf
-** $Id: error.c,v 1.135 2013/07/22 13:45:12 seiji Exp seiji $
+** $Id: error.c,v 1.139 2015/05/23 14:30:12 seiji Exp seiji $
 **	out_area_out:	error character warning generation
 **	bogus_out:	same as above
 **
@@ -41,10 +41,12 @@
 #include <errno.h>
 #include "convert.h"
 #include "oconv.h"
-#include "in_code_table.h"
+
+#if HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 #if	defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
-#include <locale.h>
 #include <libintl.h>
 #endif
 
@@ -54,12 +56,16 @@
 #include <strings.h>
 #endif
 
+#ifdef HELP_OUTPUT_HELP_OUTPUT
 #define skfstderr	stderr
+#else
+#define skfstderr	stdout
+#endif
 
 /* ident */
 extern char	*rev;
 static const char	*cpyr = 
-"Copyright (c) S.Kaneko, 1993-2013. All rights reserved.\n";
+"Copyright (c) S.Kaneko, 1993-2015. All rights reserved.\n";
 
 /* error message text   */
 static char	*skf_int_err  = 
@@ -75,6 +81,7 @@ static int	locale_enable = 0;
 static int	check_format_flaw P_((const char *));
 
 /* --------------------------------------------------------------- */
+/*@-mustfreefresh@*/
 static char *skf_gettext(in_str)
 const char	*in_str;
 {
@@ -90,6 +97,7 @@ const char	*in_str;
 #else
 #define skf_gettext(x)	x
 #endif
+/*@+mustfreefresh@*/
 
 /* --------------------------------------------------------------- */
 /*@-formatconst@*/
@@ -212,6 +220,8 @@ void out_undefined(c1,reason)
 skf_ucode c1;
 int	reason;
 {
+    int	donotout = 0;
+
 #ifdef SKFDEBUG
     if ((disp_warn || force_disp_warn || (is_v_debug)) && !input_inquiry) {
 #else
@@ -255,6 +265,7 @@ skf_gettext("skf: code(%8x) is not within UTF-32 area\n");
 skf_gettext("skf: this codeset output is not supported - ");
 	    fprintf(skfstderr,skf_err_msg);
 	    skf_outcode_display(); fprintf(skfstderr,"\n");
+	    donotout = 1;	/* could not handle output properly. */
 	    break;
 	case SKF_UNSUPP:
 	    skf_err_msg = 
@@ -270,19 +281,21 @@ skf_gettext("skf: this code point(u+%04x) is not supported\n");
 	    skf_err_msg = 
 skf_gettext("skf: encoding and output codeset is inconsistent\n");
 	    fprintf(skfstderr,skf_err_msg);
+	    donotout = 1;	/* could not handle output properly. */
 	    break;
 	case SKF_OUT_PROHIBIT:
 	    skf_err_msg = 
 skf_gettext("skf: this code point(u+%04x) output is prohibited\n");
 	    fprintf(skfstderr,skf_err_msg,c1);
-	    return;	/* this one is not error */
-	default:
+	    break;
+	default:	/* various dirty errors .... */
 	    skf_err_msg = skf_gettext(skf_int_err);
 	    fprintf(skfstderr,skf_err_msg,c1);
+	    donotout = 1;	/* could not handle output properly. */
       };
     };
 #ifndef	UNDEF_NOCLASH
-    if ((ucode_undef != 0) && !(is_o_encode) && (c1 >= 0)) {
+    if ((ucode_undef != 0) && !(is_o_encode) && (c1 >= 0) && (donotout == 0)) {
 	    post_oconv(ucode_undef);
     } else if (c1 >= 0) {
 	    post_oconv('.');
@@ -381,52 +394,42 @@ skf_gettext("skf: this codeset output is not supported - ");
 void display_help()
 {
 #if defined(SWIG_EXT)
-#ifdef NKF_COMPATMODE
     if (is_nkf_compat) {
 	printf("Usage:\tskf\t%s [--] [file]...\n\n",
     		"[-aefghjmnsvwxzAEFIJLMSWXZ] [extended_option] ");
     } else {
 	printf("Usage:\tskf\t%s [--] [file]...\n\n",
-    		"[-aefhjnsvxzAEFINSXYZ] [extended_option] ");
+    		"[-aefhjnsvwxzAEFINSXYZ] [extended_option] ");
     };
 #else
-	printf("Usage:\tskf\t%s [--] [file]...\n\n",
-    		"[-aefhjnsvxzAEFINSXYZ] [extended_option] ");
-#endif /* NKF_COMPATMODE */
-#else
-#ifdef NKF_COMPATMODE
     if (is_nkf_compat) {
 	printf("Usage:\tskf\t%s [--] [file]...\n\n",
-		"[-abefghjmnqsuvwxzAEIJLMSWXZ] [extended_option] ");
+		"[-abefghjmnqsuvwxzAEIJLMQSWXZ] [extended_option] ");
     } else {
 	printf("Usage:\tskf\t%s [--] [file]...\n\n",
-		"[-abefhjnsuvxzAEFINSXYZ] [extended_option] ");
+		"[-abefhjnsuvwxzAEFINSXYZ] [extended_option] ");
     };
-#else
-    printf("Usage:\tskf\t%s [--] [file]...\n\n",
-		"[-abefhjnsuvxzAEFINSXYZ] [extended_option] ");
-#endif /* NKF_COMPATMODE */
 #endif	/* SWIG_EXT */
     skf_err_msg = skf_gettext(
-"\tj,n\tOutout code is JIS 7/8 bit\n");
+"\tj\tOutout code is JIS 7/8 bit\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
-"\ts,x\tOutput code is Shift JIS\n");
+"\ts\tOutput code is Shift JIS\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
-"\te,a\tOutput code is EUC-JP\n");
+"\te\tOutput code is EUC-JP\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
 "\tz  \tOutput code is Unicode(TM)(UTF-8)\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
-"\tS,X\tinput character codeset is set to Shift JIS\n");
+"\tS\tinput character codeset is set to Shift JIS\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
-"\tE,A\tinput character codeset is set to EUC\n");
+"\tE\tinput character codeset is set to EUC\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
-"\tN\tinput character codeset is set to JIS 8bit\n");
+"\tJ\tinput character codeset is set to JIS 8bit\n");
     printf(skf_err_msg);
     skf_err_msg = skf_gettext(
 "\tZ\tinput character codeset is set to Unicode(TM)(UTF-8)\n");
@@ -462,8 +465,18 @@ int mode;
     fprintf(skfstderr,"%s%s",rev,cpyr);
     skf_err_msg = skf_gettext("Default input code:%s   ");
     fprintf(skfstderr,skf_err_msg,i_codeset[DEFAULT_I].desc);
-    skf_err_msg = skf_gettext("Default output code:%s   \n");
+    skf_err_msg = skf_gettext("Default output code:%s ");
     fprintf(skfstderr,skf_err_msg,i_codeset[DEFAULT_O].desc);
+#ifdef DEFAULT_EOL_CRLF
+    fprintf(skfstderr,"(CRLF)");
+#else
+#ifdef DEFAULT_EOL_LF
+    fprintf(skfstderr,"(LF)");
+#else
+    fprintf(skfstderr,"(CR)");
+#endif
+#endif
+    (void)fputc('\n',skfstderr);
 
 #ifdef SKFDEBUG
   if (is_v_debug || (mode > 0)) {
@@ -541,8 +554,11 @@ int mode;
     fprintf(skfstderr,"UID ");
 #endif
 #endif
-#ifdef HAVE_MKSTEMP
-    fprintf(skfstderr,"OMST ");
+#ifdef ENABLE_DISABLED
+    fprintf(skfstderr,"ED ");
+#endif
+#ifdef SKF_EXPERIMENTAL
+    fprintf(skfstderr,"EXP ");
 #endif
     (void)fputc('\n',skfstderr);
   } else;
@@ -570,15 +586,6 @@ int mode;
 #ifdef ACE_SUPPORT
     fprintf(skfstderr,"ACE ");
 #endif
-#ifdef NKF_COMPATMODE
-    fprintf(skfstderr,"NKF ");
-#endif
-#ifdef OLD_NKF_COMPAT
-    fprintf(skfstderr,"ONKF ");
-#endif
-#ifdef SUPPORT_HOOKED_BS
-    fprintf(skfstderr,"MAN ");
-#endif
 #ifdef FOLD_SUPPORT
     fprintf(skfstderr,"FD ");
 #ifdef SUPERFOLD
@@ -605,6 +612,24 @@ int mode;
 #endif
     } else;
 #endif
+    if (is_nkf_compat) {
+    	fprintf(skfstderr,"NKFOPT: ");
+#ifdef MIME_DECODE_DEFAULT
+	fprintf(skfstderr,"MIME_DECODE ");
+#endif
+#ifdef X0201_DEFAULT
+	fprintf(skfstderr,"X0201_DEFAULT ");
+#endif
+#ifdef HELP_OUTPUT_HELP_OUTPUT
+	fprintf(skfstderr,"SKFSTDERR ");
+#else
+	fprintf(skfstderr,"STDOUT ");
+#endif
+#ifdef SHIFTJIS_CP932
+	fprintf(skfstderr,"SJIS_IS_CP932 ");
+#endif
+	fprintf(skfstderr,"\n");
+    } else;
 #ifdef SKFDEBUG
     if (mode > 1) {
     	debug_opt_save = debug_opt;
@@ -621,6 +646,49 @@ int mod;
     display_version_common(mod);
 }
 /* --------------------------------------------------------------- */
+void display_nkf_help()
+{
+    skf_err_msg = skf_gettext(
+    	"nkf-help:\n following options are compatible\n");
+    (void)fputs(skf_err_msg, skfstderr);
+    fprintf(skfstderr,"  -j/J -s/S -e/E -b/u -f/F -L -d -c -r -m -M -h -t -v\n");
+    fprintf(skfstderr,"  --fj --unix --mac --msdos --windows --jis --euc --sjis --jis-input\n");
+    fprintf(skfstderr,"  --euc-input --sjis-input --in-place --overwrite\n");
+    skf_err_msg = skf_gettext(
+    	" * following options are enabled by default in skf\n  -I -X\n");
+    (void)fputs(skf_err_msg, skfstderr);
+    skf_err_msg = skf_gettext(
+    	" * left side nkf options and right side skf options are compatible\n");
+    (void)fputs(skf_err_msg, skfstderr);
+    skf_err_msg = skf_gettext(" nkf option        skf option\n");
+    (void)fputs(skf_err_msg, skfstderr);
+    fprintf(skfstderr," ----------------- ----------------------------------------------\n");
+    fprintf(skfstderr," -w                -z\n");               
+    fprintf(skfstderr," -W                -Z\n");               
+    fprintf(skfstderr," -w16L             --oc=utf-16le\n");               
+    fprintf(skfstderr," -W16L             --ic=utf-16le\n");               
+    fprintf(skfstderr," -w32L             --oc=utf-32le\n");               
+    fprintf(skfstderr," -W32L             --ic=utf-32le\n");               
+    fprintf(skfstderr," -Z0               --enable-ascii-conv\n");
+    fprintf(skfstderr," -Z1               --enable-ascii-conv --enable-space-conv\n");
+    fprintf(skfstderr," -Z2               --enable-ascii-conv --enable-space-sinble-conv\n");
+    fprintf(skfstderr," -Z3               --enable-ascii-conv --html-sanitize\n");
+    fprintf(skfstderr," -l                --ic=iso-2022-jp-2 --oc=iso-2022-jp-2\n");
+    fprintf(skfstderr," -x                --kana-enable\n");
+    fprintf(skfstderr," --cap-input       --decode=cap\n");
+    fprintf(skfstderr," --url-input       --decode=url\n");
+    fprintf(skfstderr," --numchar-input   --decode=hex\n");
+    fprintf(skfstderr," --guess=1         -g\n");
+    fprintf(skfstderr," --guess=2         --hard-inquiry\n");
+    fprintf(skfstderr," --ic=utf-8-bom    -z --enable-endian-mark\n");
+    fprintf(skfstderr," ----------------- ----------------------------------------------\n");
+    skf_err_msg = skf_gettext(
+    	" * following options are not supported: -i -o -T --prefix --fb-skip \n");
+    (void)fputs(skf_err_msg, skfstderr);
+    fprintf(skfstderr,"   --fb-java --no-cp932ext --no-best-fit-chars --ic=utf8-mac\n");
+    return;
+}
+/* --------------------------------------------------------------- */
 void error_code_option(code)
 int	code;
 {
@@ -648,6 +716,7 @@ int	code;
     return;
 }
 
+/*@-branchstate@*/
 void error_extend_option(skferrno,cp)
 int  skferrno;
 char *cp;
@@ -655,8 +724,9 @@ char *cp;
     if (cp == NULL) cp = "UNKNOWN";
     switch (skferrno) {
 	case SKF_NKFINCOMPAT:
-    skf_err_msg = skf_gettext("Sorry, this option(%s) is not supported by skf.\n");
-    fprintf(skfstderr,skf_err_msg,cp);
+	    skf_err_msg = skf_gettext(
+"skf: this option(%s) is not supported by skf.\n");
+	    fprintf(skfstderr,skf_err_msg,cp);
 	break;
 	case SKF_UNDEFCARGS:
 	    skf_err_msg = skf_gettext(
@@ -683,32 +753,42 @@ char *cp;
 #endif
     return;
 }
+/*@+branchstate@*/
 
 /* --------------------------------------------------------------- */
+/*@-mustfreefresh@*/
 void initialize_error()
 {
 #if	defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
     char *bt,*tl;
-    if (setlocale(LC_ALL,"") != NULL) {
-	bt = bindtextdomain(PACKAGE,LOCALEDIR);
-	tl = textdomain(PACKAGE);
-	if ((bt == NULL) || (tl == NULL)) {
+    if (skf_err_buf == NULL) {
+	if (setlocale(LC_ALL,"") != NULL) {
+	    bt = bindtextdomain(PACKAGE,LOCALEDIR);
+	    tl = textdomain(PACKAGE);
+	    if ((bt == NULL) || (tl == NULL)) {
+		skferr(SKF_MALLOCERR,(long)0,(long)0);
+		skf_exit(EXIT_FAILURE);
+	    };
+	    locale_enable = 1;
+	};
+	skf_err_buf = (char *)calloc(256,sizeof(char));
+	if (skf_err_buf == NULL) {
 	    skferr(SKF_MALLOCERR,(long)0,(long)0);
 	    skf_exit(EXIT_FAILURE);
 	};
-	locale_enable = 1;
-    };
-#endif
-    skf_err_buf = (char *)calloc(256,sizeof(char));
+    } else;
+#else
     if (skf_err_buf == NULL) {
-	skferr(SKF_MALLOCERR,(long)0,(long)0);
-	skf_exit(EXIT_FAILURE);
-    };
-#ifdef SWIG_EXT
-    skf_swig_result = SKF_MALLOCERR;
+	skf_err_buf = (char *)calloc(256,sizeof(char));
+	if (skf_err_buf == NULL) {
+	    skferr(SKF_MALLOCERR,(long)0,(long)0);
+	    skf_exit(EXIT_FAILURE);
+	};
+    } else;
 #endif
     return;
 }
+/*@+mustfreefresh@*/
 
 /* --------------------------------------------------------------- */
 /* skferr: various serious error				   */
@@ -756,7 +836,7 @@ long	a1,a2;
 		g3_table_mod->desc);
 	fprintf(skfstderr,"\n low_table:%08lx\n",(unsigned long)low_table);
 	fprintf(skfstderr," up_table:%08lx\n",(unsigned long)up_table);
-    } else if (code >= SKF_TABLEERR_G0A) {
+    } else if (code >= SKF_TABLEERR_G0) {
       switch(code) {
 	case SKF_TABLEERR_G0A:
 	    skf_err_msg = skf_gettext(
@@ -782,7 +862,7 @@ long	a1,a2;
 	case SKF_OBUFREERR:
 	    fprintf(skfstderr,"re-");		/*@FALLTHROUGH@*/
 	case SKF_OBUFERR:	
-	    fprintf(skfstderr,"obuf");		/*@FALLTHROUGH@*/
+	    fprintf(skfstderr,"obuf ");		/*@FALLTHROUGH@*/
 	case SKF_MALLOCERR:
 	    skf_err_msg = skf_gettext("failed to allocate buffer(%d)\n");
 	    fprintf(skfstderr,skf_err_msg,code);
@@ -831,7 +911,6 @@ long	a1,a2;
 #ifdef SWIG_EXT
     skf_swig_result = code;
 #endif
-    skf_exit(EXIT_FAILURE);
 }
 
 /* --------------------------------------------------------------- */
@@ -970,16 +1049,19 @@ void	skf_outcode_display()
 {
 /* name and language of the codeset */
     if ((out_codeset > 0) && (out_codeset < codeset_end)) {
-    	out_codeset = DEFAULT_O;
-    } else;
-    fprintf(skfstderr,"%s (#%d,%x%x,typ:%lx) ",
+	fprintf(skfstderr,"%s (#%d,%x%x,typ:%lx) ",
 	    i_codeset[out_codeset].desc,out_codeset,
 	    ((i_codeset[out_codeset].oconv_lang) >> 8) & 0x7f,
 	    (i_codeset[out_codeset].oconv_lang) & 0x7f,
 	    i_codeset[out_codeset].oconv_type);
+    } else {
+	skf_err_msg = skf_gettext("Unknown(internal error)");
+	(void)fputs(skf_err_msg,skfstderr);
+    };
     (void)fflush(skfstderr);
 }
 /* --------------------------------------------------------------- */
+/*@-branchstate@*/
 void test_support_charset()
 {
     int i,j;
@@ -994,7 +1076,6 @@ void test_support_charset()
 
     (void)fflush(skfstderr); (void)fflush(stdout);
     for (i=0; iso_ubytedef_table[i].ientry != NULL; i++) { 
-    	if (i == ub_ovlay_index) continue;
 	fprintf(skfstderr,"# %s:\n",iso_ubytedef_table[i].desc);
 	uentry = iso_ubytedef_table[i].ientry;
 	for (j=0; uentry[j].defschar != 0; j++) {
@@ -1019,7 +1100,7 @@ void test_support_charset()
 	      } else;
 	    } else {
 #if defined(DYNAMIC_LOADING) && !defined(SWIG_EXT)
-		if (load_external_table(&(uentry[j])) == 0) {
+		if (load_external_table(&(uentry[i])) == 0) {
 		  if (uentry[j].desc != NULL)
 		    fprintf(skfstderr,"%s%s%s*\n",
 		    skf_cname,pad,uentry[j].desc);
@@ -1058,8 +1139,10 @@ void test_support_codeset()
     };
 	trademark_warn();
 }
+/*@+branchstate@*/
 /* --------------------------------------------------------------- */
 #ifdef SKFDEBUG
+/*@-mustfreefresh@*/
 void debug_analyze()
 {
     char *trans_txt;
@@ -1165,9 +1248,7 @@ void debug_analyze()
 		fprintf(skfstderr,"UCS-2 little endian output ");
 	if (is_ucs_utf8(conv_cap)) 
 		fprintf(skfstderr,"UTF-8 little endian output ");
-#ifdef NKF_COMPATMODE
 	if (is_nkf_compat) fprintf(skfstderr,"nkf_CMPT ");
-#endif
 #ifdef KUNIMOTO
 	if (kuni_opt) fprintf(skfstderr,"LW_DET");
 #endif
@@ -1191,6 +1272,7 @@ void debug_analyze()
 	fprintf(skfstderr,"\n");
     }; 
 }
+/*@+mustfreefresh@*/
 #endif
 
 /* --------------------------------------------------------------- */
