@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 1999-2010 Seiji Kaneko. All rights reserved.
+** Copyright (c) 1999-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     esc_process.c	escape sequence handler
-    $Id: esc_process.c,v 1.7 2013/12/19 15:06:35 seiji Exp seiji $
+    $Id: esc_process.c,v 1.11 2015/04/26 15:08:38 seiji Exp seiji $
 */
 
 #include <stdio.h>
@@ -33,6 +33,11 @@
 #include "convert.h"
 #include "in_code_table.h"
 
+#define  has_valid_ctable(x) (((x)->defschar != 0) && \
+		((((x)->char_width <= 2) && ((x)->unitbl != NULL)) || \
+		 ((x)->uniltbl != NULL) || \
+		 (load_external_table(x) == 0)))
+
 static unsigned short	in_depth = FALSE;  /* recursion stopper?   */
 
 #define	ascii_index	3
@@ -50,14 +55,18 @@ static unsigned long	codeset_fl_sr_save;
 /* In this routine. skf detects Han and Chinese, but does nothing. */
 /*  if someone will write some remedy to handle these ...	   */
 /* --------------------------------------------------------------- */
-
+/*@-globstate@*//*@-usereleased@*/
 int esc_process(f)	/* escape parsing process.		   */
     skfFILE *f;
 {
     int c1,c3 = 0;      /* does not pass to/from main routine      */
     int c2;
     int idx;		/* table search index			   */
+    struct iso_byte_defs *g_table_mod;
 
+#ifdef SKFDEBUG
+    if (is_vv_debug) fprintf(stderr,"-E-");
+#endif
     if ((c1 = vGETC(f)) == sEOF) {  /* case: esc-eof		   */
 		;		/* pass esc may screw up terminal  */
 /* --------------------------------------------------------------- */
@@ -76,20 +85,17 @@ int esc_process(f)	/* escape parsing process.		   */
 		/* this sequence set char_set to G0		   */
 		/* actually, only @,A,B,C,D is allowed, but not	   */
 		/* explicitly checked here.			   */
-#ifdef NKF_COMPATMODE
 	    if (is_nkf_jfbroken) {
 		    c1 = 'B';	/* JIS X 0208			   */
 	    } else;
-#endif
 	    idx = 0;
 	    while ((iso_3_dblbyte_defs[idx].defschar != 0x00) &&
 		   (iso_3_dblbyte_defs[idx].defschar != c1)) {
 		   idx++;
 	    };
+
 	    if ((iso_3_dblbyte_defs[idx].defschar == c1) &&
-	        ((iso_3_dblbyte_defs[idx].unitbl != NULL) 
-	         || ((load_external_table(
-		    &(iso_3_dblbyte_defs[idx]))) == 0))) {
+	    	 has_valid_ctable(&(iso_3_dblbyte_defs[idx]))) {
 		g0_table_mod = &(iso_3_dblbyte_defs[idx]);
 #ifndef	SUPPRESS_FJ_CONVENSION
 		g0table2low();
@@ -143,23 +149,21 @@ int esc_process(f)	/* escape parsing process.		   */
 /* --------------------------------------------------------------- */
 /* 	case: ESC $ 2/8,2/9,2/10,2/11 F				   */
 /* --------------------------------------------------------------- */
-	    } else if ((c3 >= 0x28) && (c3 <= 0x2b)) {
+	    } else if ((c1 >= A_AT) && (c3 >= 0x28) && (c3 <= 0x2b)) {
 		idx = 0;
-#ifdef NKF_COMPATMODE
 		if (is_nkf_jfbroken) {
 		    c1 = 'B';	/* JIS X 0208			   */
 		} else;
-#endif
 		while ((iso_4_dblbyte_defs[idx].defschar != 0x00) &&
 		       (iso_4_dblbyte_defs[idx].defschar != c1)) {
 		       idx++;
 		};
+		g_table_mod = &(iso_4_dblbyte_defs[idx]);
+
 		if ((iso_4_dblbyte_defs[idx].defschar == c1) &&
-		    ((iso_4_dblbyte_defs[idx].unitbl != NULL) 
-		     || ((load_external_table(
-			&(iso_4_dblbyte_defs[idx]))) == 0))) {
+		    has_valid_ctable(&(iso_4_dblbyte_defs[idx]))) {
 		  if (c3 == 0x28) {	/* to G0		   */
-		    g0_table_mod = &(iso_4_dblbyte_defs[idx]);
+		    g0_table_mod = g_table_mod;
 #ifndef	SUPPRESS_FJ_CONVENSION
 		    if (!in_left_locking_shift) {
 			g0table2low();
@@ -232,6 +236,64 @@ int esc_process(f)	/* escape parsing process.		   */
 #ifndef	SUPPRESS_FJ_CONVENSION
 			g0table2low();
 #endif
+#ifdef SKFDEBUG
+			if (is_vv_debug) fprintf(stderr,
+		    " return to ascii due to undef set: $,(,%2x",c1);
+#endif
+		    };
+		    res_all_shift;
+		};
+	    } else if ((c1 >= A_0) && (c3 >= 0x28) && (c3 <= 0x2b)) {
+	    	/* iso-2022 private area: currently only to G0 is allowed */
+		idx = 0;
+		while ((priv_dblbyte_defs[idx].defschar != 0x00) &&
+		       (priv_dblbyte_defs[idx].defschar != c1)) {
+		       idx++;
+		};
+		if ((priv_dblbyte_defs[idx].defschar == c1) &&
+		    has_valid_ctable(&(priv_dblbyte_defs[idx]))) {
+		  if (c3 == 0x28) {	/* to G0		   */
+		    g0_table_mod = &(priv_dblbyte_defs[idx]);
+#ifndef	SUPPRESS_FJ_CONVENSION
+		    if (!in_left_locking_shift) {
+			g0table2low();
+		    };
+#endif
+		    if ((g0_table_mod->lang) != L_NU) {
+			if (!(skf_is_strong_lang(skf_input_lang))) {
+			    skf_input_lang = g0_table_mod->lang;
+			    if (output_lang == 0) {
+				skf_output_lang = skf_input_lang;
+				show_lang_tag();
+			    }; 
+			}; 
+		    };
+#ifdef SKFDEBUG
+		    if (is_vv_debug)
+			fprintf(stderr," priv-mB: $,(,%2x - %s",c1,
+				g0_table_mod->desc);
+#endif
+		  } else {
+#ifndef	SUPPRESS_UNRECOGNIZED_SEQ
+		    if (is_jiscat(conv_cap)) {
+			ox_ascii_conv(A_ESC); ox_ascii_conv('$');
+			ox_ascii_conv(c3); ox_ascii_conv(c1); 
+		    };
+#endif
+		    res_all_shift;
+		  };
+		} else {	/* UNKNOWN: pass as it is	   */
+#ifndef	SUPPRESS_UNRECOGNIZED_SEQ
+		    if (is_jiscat(conv_cap)) {
+			ox_ascii_conv(A_ESC); ox_ascii_conv('$');
+			ox_ascii_conv(c3); ox_ascii_conv(c1); 
+		    };
+#endif
+		    if (c3 == 0x28) {	/* G0 only process	   */
+			g0_table_mod = &(iso_unibyte_defs[ascii_index]);
+#ifndef	SUPPRESS_FJ_CONVENSION
+			g0table2low();
+#endif
 #ifdef SKFDEBUG
 			if (is_vv_debug) fprintf(stderr,
 		    " return to ascii due to undef set: $,(,%2x",c1);
@@ -266,6 +328,36 @@ int esc_process(f)	/* escape parsing process.		   */
 	    } else {	/* stripinvis				   */
 		c1 = seq_sweep(f,FALSE); res_all_shift;
 	    };  
+	} else if (c1 >= A_0) {
+	    /* iso-2022 private area: currently only to G0 is allowed */
+	    idx = 0;
+	    while ((priv_dblbyte_defs[idx].defschar != 0x00) &&
+		   (priv_dblbyte_defs[idx].defschar != c1)) {
+		   idx++;
+	    };
+	    if ((priv_dblbyte_defs[idx].defschar == c1) &&
+	    	has_valid_ctable(&(priv_dblbyte_defs[idx]))) {
+		g0_table_mod = &(priv_dblbyte_defs[idx]);
+#ifndef	SUPPRESS_FJ_CONVENSION
+		if (!in_left_locking_shift) {
+		    g0table2low();
+		};
+#endif
+		if ((g0_table_mod->lang) != L_NU) {
+		    if (!(skf_is_strong_lang(skf_input_lang))) {
+			skf_input_lang = g0_table_mod->lang;
+			if (output_lang == 0) {
+			    skf_output_lang = skf_input_lang;
+			    show_lang_tag();
+			}; 
+		    }; 
+		};
+#ifdef SKFDEBUG
+		if (is_vv_debug)
+		    fprintf(stderr," priv-mB: $,(,%2x - %s",c1,
+			    g0_table_mod->desc);
+#endif
+	    };
 	} else if (!stripinvis) { /* some unknown seq. 		   */
 		c1 = seq_sweep(f,FALSE);
 	} else {	/* stripinvis				   */
@@ -284,20 +376,16 @@ int esc_process(f)	/* escape parsing process.		   */
 	    in_undefined('4',SKF_UNEXPEOF);
 	    res_all_shift;
 	} else if ((c1 >= A_AT) && (c1 < A_DEL)) { /* unibyte seq. */
-#ifdef NKF_COMPATMODE
 	    if (is_nkf_jfbroken) {
 		c1 = 'B';	/* force setting to ascii	   */
 	    } else;
-#endif
 	    idx = 0;
 	    while ((iso_unibyte_defs[idx].defschar != 0x00) &&
 		   (iso_unibyte_defs[idx].defschar != c1)) {
 		   idx++;
 	    };
 	    if ((iso_unibyte_defs[idx].defschar == c1) &&
-	        ((iso_unibyte_defs[idx].unitbl != NULL) 
-	         || ((load_external_table(
-		    &(iso_unibyte_defs[idx]))) == 0))) {
+	        has_valid_ctable(&(iso_unibyte_defs[idx]))) {
 	      if (c2 == 0x28) {		/* to G0		   */
 		g0_table_mod = &(iso_unibyte_defs[idx]);
 #ifndef	SUPPRESS_FJ_CONVENSION
@@ -387,9 +475,7 @@ int esc_process(f)	/* escape parsing process.		   */
 		   idx++;
 	    };
 	    if ((iso_im2byte_defs[idx].defschar == c1) && 
-		    ((iso_im2byte_defs[idx].unitbl != NULL) 
-		     || ((load_external_table(
-			&(iso_im2byte_defs[idx]))) == 0))) {
+	        has_valid_ctable(&(iso_im2byte_defs[idx]))) {
 	      if (c2 == 0x28) {		/* to G0		   */
 		g0_table_mod = &(iso_im2byte_defs[idx]);
 #ifndef	SUPPRESS_FJ_CONVENSION
@@ -486,9 +572,7 @@ int esc_process(f)	/* escape parsing process.		   */
 		   idx++;
 	    };
 	    if ((iso_iso8859_defs[idx].defschar == c1) &&
-		((iso_iso8859_defs[idx].unitbl != NULL) 
-		 || ((load_external_table(
-		    &(iso_iso8859_defs[idx]))) == 0))) {
+	        has_valid_ctable(&(iso_iso8859_defs[idx]))) {
 		if (c2 == 0x2d) {
 		    g1_table_mod = &(iso_iso8859_defs[idx]);
 #ifndef	SUPPRESS_FJ_CONVENSION
@@ -564,7 +648,6 @@ int esc_process(f)	/* escape parsing process.		   */
 	    } else {
 		res_all_shift;
 	    };
-	    codeset_stack_depth = 0;
 	} else if (c1 == 0x47) { /* to UTF-8 with return	   */
 	    if (codeset_stack_depth == 0) {
 		in_codeset_sr_save = in_codeset;
@@ -584,7 +667,7 @@ int esc_process(f)	/* escape parsing process.		   */
 	    if (((c3 = vGETC (f)) == 0)) { /* padding  */
 		while (skip_x221_2_null() == 0) { c3 = vGETC(f); };
 	    };
-	    if (c3 == sEOF) {	/* case: esc-%-?-eof		   */
+	    if (c1 == sEOF) {	/* case: esc-%-?-eof		   */
 		in_undefined('u',SKF_UNEXPEOF); /* Do nothing	   */
 	    } else if (c3 == '@') {	/* 2Octet Level-1 ISO10646 */
 		in_codeset = codeset_utf16be; /* point of no-return   */
@@ -626,38 +709,6 @@ int esc_process(f)	/* escape parsing process.		   */
 		fprintf(stderr," 2022-extcode: %2x,%2x,%2x",'%',c1,c3);
 #endif
 /* --------------------------------------------------------------- */
-/* Coding system change(2) : JIS X-0221 use this sequence	   */
-/* --------------------------------------------------------------- */
-    } else if ((c1 == '%') && (coding_system_sense)) {
-				/* code extension method select	   */
-	if ((c1 = vGETC(f)) == sEOF) {	/* case: esc-2/5-eof	   */
-	    in_undefined('x',SKF_UNEXPEOF);
-	    res_all_shift;
-	} else if (c1 == '@') { /* standard return. do nothing	   */
-#ifdef SKFDEBUG
-	    if (is_vv_debug)
-		fprintf(stderr," codesw: %c,%2x - ",'%',c1);
-#endif
-	    if (codeset_stack_depth > 0) { /*@-globstate@*/
-		in_codeset = in_codeset_sr_save;
-		encode_cap = encode_cap_sr_save;
-		ucod_flavor = ucod_fl_sr_save;
-		codeset_flavor = codeset_fl_sr_save;
-		clear_after_mime();
-		return(sOCD);
-	    } else {
-		in_undefined('%',SKF_UNSUPP);
-		res_all_shift;
-	    };
-	} else { /* only coding system return is detected	   */
-#ifdef SKFDEBUG
-	    if (is_vv_debug)
-		fprintf(stderr," codesw: %c,%2x - ",'%',c1);
-#endif
-	    in_undefined('%',SKF_UNSUPP);
-	    res_all_shift;
-	};
-/* --------------------------------------------------------------- */
 /* Miscellaneous ISO-2022 sequences				   */
 /* --------------------------------------------------------------- */
     } else if (c1 == 0x21) {    /* control character call sequence */
@@ -670,10 +721,10 @@ int esc_process(f)	/* escape parsing process.		   */
 	    c1 = seq_sweep(f,FALSE);	/* bite it from stream	   */
 	} else  ;
     } else if (c1 == '&') {     /* version announce sequence	   */
-	if ((c3 = vGETC(f)) == sEOF) {	/* case: esc-&-eof         */
+	if (c1 == sEOF) {		/* case: esc-&-eof         */
 	    in_undefined('a',SKF_UNEXPEOF);
 	    res_all_shift;	/* just discard.		   */
-	} else if (c3 >= 'A') { /* X 0208(1990) or later. 	   */
+	} else if (c1 >= 'A') { /* X 0208(1990) or later. 	   */
 		/* we already assume 1990 revision, so ignore it.  */
 	    ; 			/* forget unknown one ;^)	   */
 	};
@@ -710,7 +761,7 @@ int esc_process(f)	/* escape parsing process.		   */
 #endif
 	res_single_shift; 
 	if (is_euc_cns(i_codeset[in_codeset].encode)) {
-	    if ((c1 = vGETC(f)) > A_SP) {  /* valid codeset	   */
+	    if ((c1 = vGETC(f)) < 0) {	/* valid codeset	   */
 		if ((c1 >= 0xa1) && (c1 <= 0xa0 + cns11643_defined_code)) {
 		    set_ss2;
 		    g2_table_mod = 
@@ -839,8 +890,7 @@ int	out;		/* out: output character if TRUE	   */
 	    /* --- note: actual termination char is > 0x30, but	   */
 	    /* 0x30-0x3f is reserved for private purpose, and not  */
 	    /* likely to be contained in standard network message. */
-	    	if (out) ox_ascii_conv(c1); 
-		break;
+	    	if (out) ox_ascii_conv(c1); break;
 	    } else if (c1 < A_SP) {	/* or in area never happen */
 	    	/* character drop or input is corrupted		   */
 	    	if (c1 == A_ESC) {
@@ -860,7 +910,7 @@ int	out;		/* out: output character if TRUE	   */
 	};
 #ifdef SKFDEBUG
 	if (is_vv_debug) {
-	    fprintf(stderr,"!"); fflush(stderr);
+	    fprintf(stderr,"!"); (void)fflush(stderr);
 	} else;
 #endif
 	return(c1);
