@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 2000-2011 Seiji Kaneko. All rights reserved.
+** Copyright (c) 2000-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     in_converter.c	input-side converter
-    $Id: in_converter.c,v 1.171 2013/07/22 13:42:44 seiji Exp seiji $
+    $Id: in_converter.c,v 1.175 2015/04/26 15:08:38 seiji Exp seiji $
 */
 
 #include <stdio.h>
@@ -58,6 +58,10 @@ static int	gb2k_in_calc_index P_((int,int));
 
 #define hku_in_calc_index big5_in_calc_index
 
+#ifdef OFFSET_CALC_ENBL
+static int	thru_in_calc_index P_((int,int));
+#endif
+
 static int	viqr_in_calc_index P_((int,int));
 static int	vni_in_calc_index P_((int,int));
 static int	iso_in_calc_index P_((int,int));
@@ -146,23 +150,12 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
     int	   c2s,c1s;
     /* Note: e_in doesn't preserve tail BS, because no following   */
     /*  modifier character(s) is given.				   */
-#ifdef	SUPPORT_HOOKED_BS
-    int	   hooked_bs = 0;
-#endif
     unsigned long	encode;
-#ifdef NKF_COMPATMODE
     int		is_nkf_euc = FALSE;
-#endif
 
     encode = i_codeset[in_codeset].encode;
-#ifdef NKF_COMPATMODE
     if (is_nkf_compat && ((in_codeset == codeset_eucjp)
 			|| (in_codeset == codeset_jis)
-			|| (in_codeset == codeset_x0213)
-			|| (in_codeset == codeset_x213a)
-			|| (in_codeset == codeset_x0213_s)
-			|| (in_codeset == codeset_euc_213)
-			|| (in_codeset == codeset_euc_213a)
 			|| (in_codeset == codeset_cp5022x)
 			|| (in_codeset == codeset_cp50221)
 			|| (in_codeset == codeset_cp50222)
@@ -178,14 +171,6 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 			|| (in_codeset == codeset_cp50222))) {
 	return(be_in(f));
     } else;
-#endif
-
-    /* wrong state change guard */
-    if (low_table == NULL) {
-    	if (g0_table_mod == NULL)
-		g0_table_mod = &(iso_unibyte_defs[ascii_index]);
-	g0table2low();
-    } else;
 
     /* --- conversion loop --------------------------------------- */
     while (TRUE) {
@@ -201,15 +186,6 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 	    c1s = c1;
 #ifdef SKFDEBUG
 	    if (is_vv_debug) fprintf(stderr,",%02x",c1);
-#endif
-#ifdef	SUPPORT_HOOKED_BS
-	    if (hooked_bs == 1) {
-		if (c1 == '_') {
-		    hooked_bs = 2; continue;
-		} else {
-		    oconv(A_BS); hooked_bs = 0;
-		};
-	    };
 #endif
 	    if ((up_dbyte && (c1 > A_KSP)) || in_single_shift) {
 		c2 &= 0x7f; c1 &= 0x7f;
@@ -255,12 +231,6 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		/* case: is_jis c1 < A_SP			   */
 		/* 	 is_jis c1 >= DEL 			   */
 		/* 	 is_euc c1 <= DEL 			   */
-#ifdef	SUPPORT_HOOKED_BS
-		if (c1 == A_BS) {  /* dirty hack for groff-output  */
-		    hooked_bs = 1;
-		    continue;
-		};
-#endif
 		in_undefined(c2,SKF_IBROKEN);
 		if (c1 == A_ESC) { /* proceed anyway.		   */
 		    if ((c1 = esc_process(f)) < 0) return(c1);
@@ -325,7 +295,7 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 			    ((c2s == 0xf4) &&
 			     ((c1s < 0xa1) || (c1s > 0xa6)))) {
 		    /* cp932 - cp51932 conversion bug recovery */
-			k0 = 0xe000 + ((c2s - 0xf0) * 188) 
+			k0 = 0xe000 + ((c2 - 0xf0) * 188) 
 			    + (c1 - 0x40) + ((c1 < A_DEL) ? 0 : 1);
 		    } else;
 		} else if (c2s > A_KSP) {
@@ -355,13 +325,6 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 	    if (in_single_shift) {
 		shift_cond_recovery();
 	    };
-#ifdef	SUPPORT_HOOKED_BS
-	    if (hooked_bs != 0) {
-		oconv(A_BS); 
-		if (hooked_bs == 2) oconv((skf_ucode)('_'));
-		hooked_bs = FALSE;
-	    };
-#endif
 	    continue;
 	} else {	/* ***** first byte ********************** */
 	    if ((c1 = vGETC(f)) < 0) return(c1);
@@ -506,7 +469,6 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		res_single_shift; 
 		set_ss3 ;	    /* SS3 in 8bit mode		   */
 	    } else {	/* 0x80 <= c1 < 0xa0 & !is_msfam	   */
-#ifdef NKF_COMPATMODE
 		if (is_nkf_euc) {
 		    reput_c1 = c1;
 		    if ((in_codeset == codeset_eucjp) 
@@ -518,9 +480,6 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		    } else;
 		    return(s_in(f));
 		} else if ((low_table_limit <= 128) && (up_table == NULL)
-#else
-		if ((low_table_limit <= 128) && (up_table == NULL)
-#endif
 			&& (up_ltable == NULL)) {
 		    in_undefined(c1,SKF_IRGTUNDEF);
 		} else if ((up_table != NULL) && 
@@ -546,6 +505,7 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 /* --------------------------------------------------------------- */
 
 /*@-globstate@*/ /* looks strange. should check it later.	   */
+/*@-usereleased@*/ /* that should be false positive		   */
 int s_in(f)			/* sjis input			   */
 skfFILE *f;			/* can process sjis, big5 etc.	   */
 {
@@ -559,8 +519,8 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
     int	   low_hz_ss = 0;
     unsigned long  encode;
     unsigned short *kana_table = NULL;
-#ifdef	SUPPORT_HOOKED_BS
-    int	   hooked_bs = 0;
+#ifdef OFFSET_CALC_ENBL
+    int	   u_offset = 0xa1;
 #endif
     int	   low_bound = 0;
     int	   up_bound;
@@ -586,6 +546,9 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 
     reset_force_disp_warn;
     if (is_gbk(encode)) {
+#ifdef OFFSET_CALC_ENBL
+	u_offset = 0x81;
+#endif
 	low_bound = 0x81; up_bound = 0xfe;
 	if (is_gb18030(encode)) {
 	    if (gb2k_a5_tbl == NULL) {
@@ -607,6 +570,9 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 	    in_calc_index = gbk_in_calc_index;
 	};
     } else if (is_msfam(encode) || is_johab(encode)) {
+#ifdef OFFSET_CALC_ENBL
+	u_offset = 0x81;
+#endif
 	low_bound = 0x81; up_bound = 0xfe;
 	up_table_limit = g1_table_mod->table_len;
 	up_width = g1_table_mod->char_width;
@@ -652,6 +618,9 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 	    in_calc_index = ms_in_calc_index;
 	};
     } else if (is_big5(encode) || is_cp950(encode) || is_hku(encode)) {
+#ifdef OFFSET_CALC_ENBL
+	u_offset = 0xa1;
+#endif
 	low_bound = 0xa1; up_bound = 0xfe;
 	up_table_limit = g1_table_mod->table_len;
 	up_width = g1_table_mod->char_width;
@@ -662,6 +631,9 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 	};
     } else if (is_big5p(encode) || is_uhc(encode) || is_big5_uao(encode)) {
 		/* BIG5+HKSCS, UAO, UHC */
+#ifdef OFFSET_CALC_ENBL
+	u_offset = 0x81;
+#endif
 	low_bound = 0x81; up_bound = 0xfe;
 	up_table_limit = g1_table_mod->table_len;
 	up_width = g1_table_mod->char_width;
@@ -681,23 +653,19 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
     } else if (is_nniso_gsm(encode)) {
 	low_bound = 0xa1; up_bound = 0xfe;
     	in_calc_index = nniso_in_calc_index;
+#ifdef OFFSET_CALC_ENBL
+    } else if (is_thru(encode)) {
+	low_bound = 0xa1; up_bound = 0xfe;
+    	in_calc_index = thru_in_calc_index;
+#endif
     } else {
 	low_bound = 0xa1; up_bound = 0xfe;
     	in_calc_index = dmy_in_calc_index;
     };
 
-#ifdef NKF_COMPATMODE
     if (is_nkf_compat && (is_msfam(encode)) && is_nkf_rotmode) {
 	return(be_in(f));
     } else;
-#endif
-
-    /* wrong state change guard */
-    if (low_table == NULL) {
-    	if (g0_table_mod == NULL)
-		g0_table_mod = &(iso_unibyte_defs[ascii_index]);
-	g0table2low();
-    } else;
 
     while (TRUE) {
 	if (c2) {       /* ***** second byte ********************* */
@@ -717,15 +685,6 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 	    } else if (c1 < 0) return(c1); /* should be trapped    */
 #ifdef SKFDEBUG
 	    if (is_vv_debug) fprintf(stderr,",%02x",c1);
-#endif
-#ifdef	SUPPORT_HOOKED_BS
-	    if (hooked_bs == 1) {
-		if (c1 == '_') {
-		    hooked_bs = 2; continue;
-		} else {
-		    oconv(A_BS); hooked_bs = 0;
-		};
-	    };
 #endif
 	    /* there are 3 case in calc_in_index process.	   */
 	    /*  1. just calculate index for the codeset		   */
@@ -830,13 +789,6 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 		in_sbroken(c2,c1);
 	    };
 	    c2 = 0;
-#ifdef	SUPPORT_HOOKED_BS
-	    if (hooked_bs) {
-		oconv(A_BS); 
-		if (hooked_bs == 2) oconv((skf_ucode)('_'));
-		hooked_bs = FALSE;
-	    };
-#endif
 	} else {	/* ***** first byte ********************** */
 			/* Note: do not return EOF to reput_c1	   */
 	    if (reput_c1 >= 0) { 
@@ -881,6 +833,10 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 			c2 = c1; continue;
 		    } else if ((c1 = esc_process(f)) < 0) return(c1);
 		    if (low_table == NULL) low_table = ascii_uni_byte;
+		    if (c1 > A_DEL) {
+		    	c2 = c1;
+		    	continue;
+		    } else;
 		} else if (c1 == A_SI) {  /* shift-in control      */
 		    g0table2low();
 		    res_locking_shift;
@@ -892,7 +848,7 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 		};
 		/* --- ASCII alphanumeric area ------------------- */
 	    } else if (c1 <= A_DEL) {	/* 0x21-0x7f               */
-		if (low_dbyte) { 
+		if (low_dbyte) {
 		    c2 = c1; continue;
 		} else if ((c1 == '~') && is_hz(encode)) {
 		    if ((c1 = vGETC(f)) < 0) {
@@ -1222,11 +1178,10 @@ skf_ucode ch;
 int	cod;
 {
     skf_ucode c4,c5;
-    int c1;
 
     if ((ch == 0xfeff) || (ch == 0xfffe)) { /* BOM		  */
 	return(0);		/* just discard			  */
-    } else if ((ch >= 0xd800) && (ch <=0xdbff)){ /* surrogate	  */
+    } if ((ch >= 0xd800) && (ch <=0xdbff)){ /* surrogate	  */
 		    /* utf7 with surrogate case		  */
 	if ((c4 = u_dec_hook(f,cod)) == sEOF) {
 	    in_undefined(c4, SKF_NOSURRG); 
@@ -1238,20 +1193,6 @@ int	cod;
 	c5 = ((ch - 0xd800) << 10) + ((c4 - 0xdc00) & 0x3ff)
 	    + 0x10000;
 	oconv(c5);
-    } else if (ch == A_ESC) {
-	if ((c4 = u_dec_hook(f,cod)) == sEOF) {
-	    in_undefined(c4, SKF_UNEXPEOF); 
-	    return(sEOF);
-	} else if ((c5 = u_dec_hook(f,cod)) == sEOF) {
-	    in_undefined(c4, SKF_UNEXPEOF); 
-	    return(sEOF);
-	} else;
-	if ((c4 == '%') && (c5 == '@')) { /* standard return	   */
-	    enque(c4); enque(c5);
-	    if ((c1 = esc_process(f)) < 0) return(c1);
-	} else {
-	    oconv(A_ESC); oconv(c4); oconv(c5);
-	};
     } else if ((ch < 0x110000) || unchk_utf32_range) {
 	oconv(ch);
     } else {
@@ -1275,11 +1216,7 @@ skfFILE *f;
 
     if (is_encode_incomp_ucs2) {
     	/* utf-16 doesn't support any encodings			   */
-#ifdef NKF_COMPATMODE
 	if (!is_nkf_compat) in_undefined(0,SKF_DECINCONS);
-#else
-	in_undefined(0,SKF_DECINCONS);
-#endif
     	decode_set(0);
     } else;
     if ((in_codeset == codeset_utf32be)
@@ -1331,11 +1268,7 @@ register skfFILE *f;		/* this never mixed with jis 	   */
 {
     if (is_encode_incomp_ucs2) {
     	/* utf7 doesn't support any encodings			   */
-#ifdef NKF_COMPATMODE
-	if (!is_nkf_compat) in_undefined(0,SKF_DECINCONS);
-#else
 	in_undefined(0,SKF_DECINCONS);
-#endif
     	decode_set(0);
     } else;
     return(uni_in(f,DEC_HOOK_UTF7));
@@ -1372,11 +1305,6 @@ register skfFILE *f;
 {
     int c1;
 
-    if (is_mimeb_encoded || is_mimeq_encoded || is_mimeb_strict 
-    	|| is_puny_encoded) {
-	decode_set(0);
-    } else;
-
     while (TRUE) {
 	if ((c1 = vGETC(f)) == sEOF) {
 	    return(sEOF);
@@ -1572,10 +1500,7 @@ unsigned long ec;	/* encode				  */
     };
 
     if (is_vni(ec)) {
-    	if (ca < 0x40) {
-	    in_sbroken(ch,ca);
-	    return(-1);
-	} else if ((ca < 0xc0) || ((cy = viq_tbl[ca - 0x40]) == 0)) {
+	if ((ca < 0xc0) || ((cy = viq_tbl[ca - 0x40]) == 0)) {
 	    oconv(ch); return(ca);
 	};
 	/* ch other than follows never comes here */
@@ -1845,6 +1770,14 @@ int c1,c2;
 #endif
 /* --------------------------------------------------------------- */
 #ifdef PART_INDEX
+#ifdef OFFSET_CALC_ENBL
+static int thru_in_calc_index(c1,c2)
+int c1,c2;
+{
+    return(c2);
+}
+#endif
+/* --------------------------------------------------------------- */
 static int johab_in_calc_index(c1,c2)
 int c1,c2;
 {
@@ -2039,7 +1972,6 @@ int low_bound,u_offset;
     } else in_undefined(c2,SKF_IBROKEN);
 }
 #endif
-#ifdef NKF_COMPATMODE
 /* --------------------------------------------------------------- */
 /*  be_in: Broken JIS and rot support				   */
 /* --------------------------------------------------------------- */
@@ -2066,26 +1998,20 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
     skf_input_lang = M_JP;
 
     encode = i_codeset[in_codeset].encode;
-    g1table2up();
-#if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) && defined(NKF_COMPATMODE)
+#if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) 
     if (nkf_compat && is_nkf_rotmode) mod = TRUE;
     else mod = FALSE;
 #endif
 
-    /* wrong state change guard */
-    if (low_table == NULL) {
-    	if (g0_table_mod == NULL)
-		g0_table_mod = &(iso_unibyte_defs[ascii_index]);
-	g0table2low();
-    } else;
-
     /* --- conversion loop --------------------------------------- */
     while (TRUE) {
 	if (c2) {       /* ***** second byte ********************* */
 	    c2s = c2;	/* save for later test			   */
 	    if ((c1 = vGETC(f)) == sEOF) {
 			/* c2 is rot'd even if c1 is EOF	   */
-		in_undefined(c2,SKF_UNEXPEOF);
+		if (!encode_enbl || !is_mime_nkfmode) {
+			in_undefined(c2,SKF_UNEXPEOF);
+		};
 		break;
 	    } else if (c1 == sOCD) return(sOCD);
 	    c1s = c1;
@@ -2094,8 +2020,7 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 #endif
 	    c3 = c2;
 	    if (is_msfam(encode)) {
-	    /* rot case: code bound does not applied		   */
-	      if ((c1 <= 0xff) && (c1 >= 0x21)
+	      if ((c1 <= 0xfc) && (c1 >= 0x40)
 			&& (c1 != 0x7f)) {
 		if ((c2 <= 0xef) || (is_ms_213c(encode) == 0)) {
 					/* can convert into JIS	   */
@@ -2202,18 +2127,16 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		};
 	    } else if (((c2s < A_DEL) && (ch < low_table_limit))
 		     || ((c2s > A_KSP) && (ch <up_table_limit))) {
-		if (is_cp5022x(encode)) {
+		if (is_euc_mseuc(encode) || is_cp5022x(encode)) {
 		    if (ch < X0208_KANJI_END) {
 			if (is_cp5022x(encode)) k0 = low_table[ch];
 			else k0 = up_table[ch];
-#if 0
 		    } else if ((((c2 >= 0x75) && (c2 < 0x79))
 			    || ((c2 >= 0x7d) && (c2 <= 0x7e)))
 			    && (c1 > A_SP)
 			    && (is_euc_mseuc(encode))) {
 			k0 = ((c2s - 0xf5) * 94)
 			    + (c1s - 0xa1) + 0xe40c;
-#endif
 		    } else if ((c2 < 0x7d) &&
 			(((c1 > A_SP) && (is_cp5022x(encode))) ||
 			(((c1s > A_KSP)
@@ -2282,11 +2205,9 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		    SKF_rotoconv(c1,mod); 
 		} else if (c1 < 0x0e) {	/* 0x00-0x0d		   */
 		    SKF_rotoconv(c1,mod);/* common control code	   */
-#ifdef NKF_COMPAT
 		    if (is_nkf_jffbroken && is_lineend(c1)) {
 		    	res_all_shift;
 		    } else;
-#endif
 #ifdef KUNIMOTO
 		    if (kuni_opt && is_lineend(c1)) {
 			res_all_shift; return(c1);
@@ -2330,14 +2251,7 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		/* --- 0xa1 <= c1 <= 0xff ------------------------ */
 	    } else if (c1 >= A_KSP) {	/* c1 >= 0xa0		   */
 			/* code is within JIS upper plane	   */
-		if (is_msfam(encode)) {
-		    if (c1 <= KANA_END) {
-			oconv(c1 + 0xff60 - 0xa0);
-	    	    } else { /* 0x80 <= c1 < 0xa0 && is_msfam	*/
-			c2 = c1; continue;
-		    };
-		} else if (is_multibyte(up_dbyte) ||
-		    ((c1 > KANA_END) && is_msfam(encode))) {
+		if (up_dbyte) {
 				/* lower plane is 2byte locked	   */
 		    c2 = c1 ;	/* pass it to second byte	   */
 		    continue;	/* goto next_byte		   */
@@ -2352,8 +2266,9 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		};
 		/* --- rest 0x80 <= c1 < 0xa0 -------------------- */
 		/* comparison is for KOI-8-R consideration	   */
-	    } else if (is_msfam(encode)) {
-	    	c2 = c1; continue;
+	    } else if ((c1_table_mod != NULL) && 
+	    		(c1_table_mod->unitbl != NULL)) {
+		if ((c1 = c1_process(f,c1)) < 0) return(c1);
 	    } else if (is_cp5022x(encode) && (c1 <= 0x98)) {
 		c2 = c1; continue;
 	    } else {	/* 0x80 <= c1 < 0xa0 & !is_msfam	   */
@@ -2374,5 +2289,4 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
     };
     return(sEOF); 
 }
-#endif
 /* --------------------------------------------------------------- */
