@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 2000-2011 Seiji Kaneko. All rights reserved.
+** Copyright (c) 2000-2014 Seiji Kaneko. All rights reserved.
 ** Copyright (c) 2003 The Internet Society. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
@@ -23,7 +23,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     in_decoder.c	input-side converter decode routines
-    $Id: in_decoder.c,v 1.69 2013/07/22 13:45:12 seiji Exp seiji $
+    $Id: in_decoder.c,v 1.71 2015/04/26 15:11:48 seiji Exp seiji $
 
     Punycode decode part is copyrighted by The Internet society.
 */
@@ -395,9 +395,9 @@ int racecode_decode(input_length, input, output_length, output)
 	    res = ((uw & 0x07U) << 5);
 	    b32stat = 3;
 	} else if (b32stat == 3) {
-	    chu = res + uv;
-	    res = 0;
-	    b32stat = 0;
+	    chu = res + ((uv >> 2) & 0xfU); b32stat = 2;
+	    res = (uv << 6) & 0xc0U;
+	    b32stat = 3;
 	} else {	/* b32stat == 0				   */
 	    if ((uw = race_decode_digit(input[i++])) < 0) {
 		return punycode_bad_input; /* should not occur	   */
@@ -409,11 +409,8 @@ int racecode_decode(input_length, input, output_length, output)
 	if (((i != 0) || (chu != 0xd8)) 
 		&& (lcheck == 0) && (wn1 == 0)) {
 	    u1 = chu; wn1 = 1;
-	    if (chu == 0xd8) {
-	    	wn1 = 8; lcheck = 1;
-	    } else;
 	} else if (wn1 == 1) {		/* Step 2		   */
-	    if (chu == 0xff) {
+	    if (uv == 0xff) {
 		wn1 = 5; continue;
 	    };
 	    if ((u1 == 0) && (chu == 0x99)) { /* Step 3 & 4	   */
@@ -423,9 +420,7 @@ int racecode_decode(input_length, input, output_length, output)
 	    *output_length += 1;
 	    wn1 = 1; continue; 
 	} else if (wn1 == 5) {		/* Step 5		   */
-	    if ((i == (input_length - 1)) && (b32stat == 0)) {
-	    	return(punycode_bad_input);
-	    } else if (chu == 0x99) {
+	    if (chu == 0x99) {
 		output[*output_length] = (u1 << 8) + 0xff;
 		*output_length += 1;
 	    } else {
@@ -440,8 +435,7 @@ int racecode_decode(input_length, input, output_length, output)
 	    *output_length += 1;
 	    lcheck = 1; continue;
 	} else {	/* LCHECK start  case			   */
-	    lcheck = 1; u1 = chu;
-	    continue; /* skip lcheck validate, for	   */
+	    lcheck = 1; continue; /* skip lcheck validate, for	   */
 			  /* such check in skf is worthless	   */
 	}; 
     };
@@ -772,7 +766,6 @@ int	*sy;
     char mime_cname[MIME_CSET_LEN];
 
     sy += 2;	/* skip '=?' part				   */
-
     /* put codeset name part into mime_cname	*/
     for (i=0;i<MIME_CSET_LEN;i++) {
 	if ((sy[i] == '\0') || (sy[i] == '?')
@@ -982,9 +975,15 @@ int	flg;
 	} else {
 	    mime_res_bit = 0; return(c1); 
 	}; 
+#if 0
       } else if (((is_mimeb_encoded) || (is_mimeb_strict) 
 		 || (is_mimeq_encoded)) 
 		 && (!is_in_ucs_ufam || in_mime)) {
+#else
+      } else if (((is_mimeb_encoded) || (is_mimeb_strict) 
+		 || (is_mimeq_encoded)) 
+		 && (!is_in_ucs_ufamnomime || in_mime)) {
+#endif
 			/* *************************************** */
 			/* MIME decoder 			   */
 			/* *************************************** */
@@ -1117,19 +1116,24 @@ int	flg;
 #ifdef SKFDEBUG
 		  if (is_vv_debug) fprintf(stderr,"-Q-softwrap ");
 #endif
-#if 1
 		  post_Qclear();
-#else
-		  post_Qunenque();
-#endif
 	          if ((c3 = hook_getc(f,flg)) == sEOF) continue;
 		  /* discard CRLF and LFCR */
 		  if ((c1 == A_LF) && (c3 == A_CR)) continue;
 		  if ((c3 == A_LF) && (c1 == A_CR)) continue;
 		  /* other case: restart at c3			   */
 		  pre_Qenque(c3);
+		  continue;
 	      };
-	      continue;
+	      if (!is_nkf_compat || !is_hex_char(c1)) continue;
+	      if ((c3 = hook_q_getc(f,flg)) == sEOF) continue;
+#ifdef SKFDEBUG
+	      if (is_vv_debug) fprintf(stderr,"-%02x ",c3);
+#endif
+	      pre_Qenque(c1); pre_Qenque(c3);
+	      if (!is_hex_char(c3)) continue;   /* broken	   */
+	      ch = (skf_hex(c1) << 4) + skf_hex(c3);
+	      post_Qclear(); pre_Qclear(); return(ch);
 	  };
 	} else if (in_mime && is_b_encode) {
 #ifdef SKFDEBUG
@@ -1173,7 +1177,7 @@ int	flg;
 		} else if ((c3 == A_LF) && 
 			((mime_mark == 2) || (mime_mark == 0))) {
 		    if (mime_mark == 0) {
-#if defined(DELETESPACEONLE) && defined(NKF_COMPATMODE) 
+#if defined(DELETESPACEONLE)
 			if (is_skf196mime && !is_nkf_compat) {
 			    pre_Qclear();
 			} else;
@@ -1185,7 +1189,7 @@ int	flg;
 		} else if ((c3 == A_CR) && 
 			((mime_mark == 1) || (mime_mark == 0))) {
 		    if (mime_mark == 0) {
-#if defined(DELETESPACEONLE) && defined(NKF_COMPATMODE)
+#if defined(DELETESPACEONLE)
 			if (is_skf196mime && !is_nkf_compat) {
 			    pre_Qclear();
 			} else;
@@ -1538,7 +1542,7 @@ int	flg;
 	    };
 	    pre_Qenque(c3);  /* re-supply delimiter	  */
 	    codeset_push_for_mime();
-	    in_codeset = codeset_utf16le;
+	    in_codeset = codeset_utf16be;
 	    set_dummy_detect;
 	    set_in_endian;		/* set big endian	   */
 	    in_ace = TRUE;
@@ -1838,9 +1842,9 @@ int cod;
 	  };
 	} else {		/* utf-16 (a.k.a. UCS2)		   */
 #ifdef SKFDEBUG
-	    if (is_vv_debug) {
+	  if (is_vv_debug) {
 	  	fprintf(stderr," U2:%d",in_codeset);
-	    } else;
+	  } else;
 #endif
 	    c2 = c1;
 	    GETC_add(c1,c2,f);
