@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 1993-2012 Seiji Kaneko. All rights reserved.
+** Copyright (c) 1993-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
 ** oconv.c:	unic*de post-handler. called from in_converter.
-** $Id: oconv.c,v 1.142 2013/12/19 15:06:35 seiji Exp seiji $
+** $Id: oconv.c,v 1.144 2015/04/26 15:08:38 seiji Exp seiji $
 **
 **********************************************************************
 ** Notice for limitation of these converter programming.
@@ -45,7 +45,7 @@
 #include <strings.h>
 #endif
 
-#if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) && defined(NKF_COMPATMODE)
+#if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) 
 static void SKFROTPUT P_((int));
 #endif
 /* --------------------------------------------------------------- */
@@ -303,53 +303,28 @@ int	oconv_init()
     if (is_o_encode) {
 /* encoding and MIME */
     /* do not set encoding if GL part is not iso-2022 like code */
-    	if (is_ucs_utf7(conv_cap) || is_brgt(conv_cap)
+    	if (is_ucs_utf7(conv_cap) || is_ucs_brgt(conv_cap)
 	  || is_ucs_uri(conv_cap)
 	  || is_ucs_transp(conv_cap)
 	  || is_hzzW(conv_cap) || is_vni(conv_cap) 
-	  || (in_codeset == codeset_transp)
 	  || is_viqr_or_vimn(conv_cap) 
 	  || is_keis(conv_cap)) {
 	  /* these output encodings do not support MIMEs	*/
 	  out_undefined(sFLSH,SKF_ENC_ERR);
 	  o_encode = 0;
-	} else if ((out_codeset == codeset_binary)
-	  || (out_codeset == codeset_locale)
-	  || (out_codeset == codeset_puny)
-	  || (out_codeset == codeset_transp)
-	  || (out_codeset == codeset_gsm0338)
-	  || (out_codeset == codeset_brgt)
-	  || (out_codeset == codeset_sjisau)
-	  || (out_codeset == codeset_sjissb)
-	  || (out_codeset == codeset_sjisontt)
-	  || (out_codeset == codeset_sjiscl)
-	  || (out_codeset == codeset_maccyrl)
-	  || (out_codeset == codeset_macCE)
-	  || (out_codeset == codeset_macroman)
-	  || (out_codeset == codeset_macdevang)
-	  || (out_codeset == codeset_macturka)
-	  || (out_codeset == codeset_macgreek)
-	  || (out_codeset == codeset_maciceln)
-	  || (out_codeset == codeset_macgujar)
-	  || (out_codeset == codeset_macgurmu)
-	  || (out_codeset == codeset_maccroatian)
-	  || (out_codeset == codeset_macromna)
-	  || (out_codeset == codeset_uri)
-	  || (out_codeset == codeset_kr)) {
-	  out_undefined(sFLSH,SKF_ENC_ERR);
-	  o_encode = 0;
 	} else;
 	if (is_o_encode_hex(o_encode) 
 		|| (is_o_encode_oct(o_encode))) 
 	    o_encode_stat = TRUE;
 	else if (is_o_encode_b64(o_encode)) 
 	    o_encode_stat = TRUE;
+	else if (is_o_encode_oct(o_encode)) 
+	    o_encode_stat = TRUE;
 	else if (is_o_encode_uri(o_encode)) 
 	    o_encode_stat = TRUE;
 /* encoding and inquiry */
 	if (input_inquiry) {
 	  o_encode = 0;
-	  o_encode_stat = FALSE;
 	};
     } else;
 
@@ -424,10 +399,8 @@ skf_ucode ch;
     int		charProp = 0;	/* character property buffer	   */
     char	kinsoku_stat;
     int		trim = 0;
-#ifdef NKF_COMPATMODE
     int		nkftrim = 0;
 #endif
-#endif
 
 #ifdef SKFDEBUG
     if (is_vv_debug) {
@@ -454,11 +427,17 @@ skf_ucode ch;
 	    	return;
 	    };
 #ifndef USE_1995L
-	    if ((ch >= 0xff61) && (ch <= 0xff9f) && 
-	        (is_nkf_no_hk ||
-		 (!hk_enbl && !(use_compat && is_ucs_ufam(conv_cap))))) {
-		(void)x0201conv((ch - 0xff40),0xb0);
-		return;
+	    if ((ch >= 0xff01) && (ch <= 0xff9f)) {
+		if ((ch >= 0xff61) && (ch <= 0xff9f) && 
+		    (is_nkf_no_hk ||
+		     (!hk_enbl && !(use_compat && is_ucs_ufam(conv_cap))))) {
+		    (void)x0201conv((ch - 0xff40),0xb0);
+		    return;
+		} else if ((ch >= 0xff01) && (ch <= 0xff5e) && 
+		    (is_nkf_no_hk || is_o_prefer_ascii)) {
+		    ch -= (0xff00 + 0x20);	/* convert to ascii	*/
+		    /* pass to later process */
+		} else;
 	    } else;
 	} else {
     /* --- Note: This is a point where all normalize packing	   */
@@ -482,8 +461,7 @@ skf_ucode ch;
 		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
 			    (u_dakuten[c1] == 3)) {
 			ch = sgbuf + 2; sgbuf_buf = 0;
-		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
-			    (u_dakuten[c1] == 4)) {
+		    } else if ((ch == U_KHDK) && (u_dakuten[c1] == 4)) {
 			switch (sgbuf) {
 			    case 0x304b: ch = 0xd801; sgbuf_buf = 0; break;
 			    case 0x30ab: ch = 0xd808; sgbuf_buf = 0; break;
@@ -591,7 +569,6 @@ skf_ucode ch;
 /* nkf -Z and various zenkaku-hankaku conversion hook */
     if (is_nkf_convert_hook) {
     /* FIXME: should test whole unicode region for --no-best-fit-chars */
-#if defined(NKF_COMPATMODE)
 	if (use_latin2null && is_no_bfc && is_nkf_compat && (
 		((ch >= 0xffe0) && (ch <= 0xffef)) 
 	     ||	((ch >= 0x80) && (ch <= 0xff)) 
@@ -600,9 +577,6 @@ skf_ucode ch;
 	     || (ch == 0x203e) || (ch == 0x2225) || (ch == 0xff0d))) {
 	     return;	/* discard */
 	} else if (is_ascii_conv && (ch >= 0xff01) && (ch <= 0xff5e)) {
-#else
-	if (is_ascii_conv && (ch >= 0xff01) && (ch <= 0xff5e)) {
-#endif
 	    ch -= 0xfee0;	/* conver to pure ascii		  */
 	} if (is_ascii_conv && (ch == 0x2212)) { /* minus hack	  */
 	    ch = 0x2d;
@@ -631,8 +605,19 @@ skf_ucode ch;
 	    c1 = ch; 
 	    if (ch == A_CR) {
 	      if (!detect_cr) {
-	      	set_detect_cr;
+#ifdef SKFDEBUG
+		if (is_vv_debug) fprintf(stderr,"-CR mixture-");
+#endif
+		set_detect_cr;
 		if (!detect_lf) set_first_detect_cr;
+#ifdef FOLD_SUPPORT
+		if (!is_lineend_normalize || !fold_fclap) {
+#else
+		if (!is_lineend_normalize) {
+#endif
+		    set_use_detect_cr;
+		    if (!use_detect_lf) set_use_first_detect_cr;
+		} else;
 	      } else;
 #ifdef FOLD_SUPPORT
 	      if (fold_fclap) {
@@ -659,31 +644,49 @@ skf_ucode ch;
 		    } else c1 = -1;	/* eliminate		  */
 		} else if (sgbuf_buf == 0) {	/* EOF or FLSH	  */
 		    SKFCRLF(); c1 = -1;
-#ifdef NKF_COMPATMODE
 		} else if (is_nkf_compat && (prev_ch > 0x2000)) {
 		    c1 = -1;	/* eliminate			  */
-#endif
 		} else if (is_sentence_clip && (fold_count == 0)) {
 		    c1 = -1;	/* eliminate			  */
 		} else {
 		    c1 = A_SP; fold_count++;
 		}; 
-	      } else fold_count = 0;
+	      } else;
 #endif	/* FOLD_SUPPORT */
-	      if (!is_lineend_thru) {
+	      if (!is_lineend_thru
+	      	|| (notrunc_le && is_lineend_normalize)) {
 		  if (!detect_lf || first_detect_cr) {
 		      SKFCRLF();
-#if defined(NKF_COMPATMODE)
-		  } else if (is_nkf_compat && detect_lf) {
+#ifdef FOLD_SUPPORT
+		  } else if (notrunc_le && is_lineend_normalize
+		  	&& (prev_ch == A_LF)) {
+		      ch = -1;		/* LF-CR: just discard		  */
+		      		/* put dummy char to prev_ch 		  */
+#endif
+#if 0
+		  } else if (is_nkf_compat && detect_lf 
+		  	&& (prev_ch != A_LF)) {
 		      SKFCRLF();
 #endif
+		  } else if (is_nkf_compat && detect_lf) {
+		      SKFCRLF();
 		  } else;
 		  c1 = -1;
 	      };
 	    } else if (c1 == A_LF) {
 	      if (!detect_lf) {
-	      	set_detect_lf;
-	      };
+#ifdef SKFDEBUG
+		if (is_vv_debug) fprintf(stderr,"-LF mixture-");
+#endif
+		set_detect_lf;
+#ifdef FOLD_SUPPORT
+		if (!is_lineend_normalize || !fold_fclap) {
+#else
+		if (!is_lineend_normalize) {
+#endif
+		    set_use_detect_lf;
+		} else;
+	      } else;
 #ifdef FOLD_SUPPORT
 	      if (fold_fclap) {
 		if (notrunc_le) {
@@ -707,21 +710,24 @@ skf_ucode ch;
 		    } else c1 = -1;	/* eliminate		  */
 		} else if (sgbuf_buf == 0) {	/* EOF or FLSH	  */
 		    SKFCRLF(); c1 = -1;
-#ifdef NKF_COMPATMODE
 		} else if (is_nkf_compat && (prev_ch > 0x2000)) {
 		    c1 = -1;	/* eliminate			  */
-#endif
 		} else if (is_sentence_clip && (fold_count == 0)) {
 		    c1 = -1;	/* eliminate			  */
 		} else {
 		    c1 = A_SP; fold_count++;
 		};
-	      } else fold_count = 0;
+	      } else;
 #endif		/* FOLD_SUPPORT */
-	      if (!is_lineend_thru) {
-#if defined(NKF_COMPATMODE)
+	      if (!is_lineend_thru
+	      	|| (notrunc_le && is_lineend_normalize)) {
 		  if (!detect_cr) {
 		      SKFCRLF();
+#ifdef FOLD_SUPPORT
+		  } else if (notrunc_le && is_lineend_normalize
+		  	&& (prev_ch == A_CR)) {
+		      ch = -1;	/* CR-LF: just discard			  */
+#endif
 		  } else if (!is_nkf_compat && !first_detect_cr) {
 		      SKFCRLF();
 #ifdef FOLD_SUPPORT
@@ -730,11 +736,6 @@ skf_ucode ch;
 		      SKFCRLF();
 #endif
 		  } else;
-#else
-		  if (!detect_cr || !first_detect_cr) {
-		      SKFCRLF();
-		  } else;
-#endif
 		  c1 = -1;
 	      };
 /* #ifdef FOLD_SUPPORT */
@@ -848,21 +849,12 @@ skf_ucode ch;
 	    } else; 
 	  };
 #endif
-#ifdef NKF_COMPATMODE
-	  if (is_nkf_compat) {
-		if ((ch >= 0x3041) && (ch <= 0x3096)) {
-		    if (is_nkf_c_katakana) ch += 0x60L;
-		} else if ((ch >= 0x30a1) && (ch <= 0x30a6)) {
-		    if (is_nkf_c_hiragana) ch -= 0x60L;
-		} else;
-	  } else;
-#else
+/* nkf-compatible katakana-hiragana-convert */
 	  if ((ch >= 0x3041) && (ch <= 0x3096)) {
 		if (is_nkf_c_katakana) ch += 0x60L;
 	  } else if ((ch >= 0x30a1) && (ch <= 0x30a6)) {
 		if (is_nkf_c_hiragana) ch -= 0x60L;
 	  } else;
-#endif
 #ifdef	FOLD_SUPPORT
 	  if (fold_fclap > 0) {
 	    if (fold_count > (fold_clap - 2)) { /* dbytes	  */
@@ -990,16 +982,10 @@ skf_ucode ch;
     } else if (ch < 0x10000) {	/* compatibility plane		  */
 #ifdef	FOLD_SUPPORT
 	if (fold_fclap > 0) {	/* if folding is enabled	  */
-#if 1
 	  if (is_nkf_convert_hook) { nkftrim = 1;
 	  } else;
-#endif
 	  trim = ((ch >= 0xff60) && (ch < 0xffe0)) ? 1 : 0;
-#ifdef NKF_COMPATMODE
 	  if (fold_count > (fold_clap - 2 + trim + nkftrim)) {
-#else
-	  if (fold_count > (fold_clap - 2 + trim)) {
-#endif
 	    if (fold_count > (fold_fclap + trim)) {
 		SKFCRLF(); fold_count = 0;
 	    } else if ((ch == 0xff08) || (ch == 0xff3b)
@@ -1058,24 +1044,6 @@ skf_ucode ch;
 		    decompose_code(uukp);
 		    return;
 		};
-	    } else if ((ch >= 0x11090) && (ch < 0x11600)) {
-	    	/* KAITHI and other codes */
-		if ((uukp = nkdc_lowptr[ch - 0x11090 + UNI_KAITHI_OFF]) != 0) {
-		    decompose_code(uukp);
-		    return;
-		};
-	    } else if ((ch >= 0x1ee00) && (ch < 0x1ef00)) {
-	    	/* arabic mathmatical addendums */
-		if ((uukp = nkdc_lowptr[ch - 0x1ee00 + UNI_ARAM_OFF]) != 0) {
-		    decompose_code(uukp);
-		    return;
-		};
-	    } else if ((ch >= 0x1f100) && (ch < 0x1f400)) {
-	    	/* Enclosed CJK addendums and emoticons */
-		if ((uukp = nkdc_lowptr[ch - 0x1f100 + UNI_ECSP_OFF]) != 0) {
-		    decompose_code(uukp);
-		    return;
-		};
 	    } else if ((ch >= 0x2f800) && (ch < 0x2fa20)) {
 			    /* CJK cpt.sup  */
 		if ((uukp = nkdc_lowptr[ch - 0x2f800 + UNI_CJKC_OFF]) != 0) { 
@@ -1626,26 +1594,18 @@ skf_ucode c3;
 	if (d2 == HZ_ANN) { SKFputc(HZ_ANN); };
 	SKFputc(d2);
     } else if (is_zW(conv_cap)) {
-	if ((d2 != A_LF) && (d2 != A_CR)) {
-	    if (is_zw_shift) {
-	    	if (d2 == A_SP) {
-		   SKFputc(ZW_LF);
-		} else SKFputc(A_SP);
-	    } else {
-		SKFputc(ZW_ANN); SKFputc(ZW_ENT); /* dirty but needed  */
-	    	if (d2 == A_SP) {
-		   SKFputc(ZW_LF);
-		} else SKFputc(A_SP);
-		set_zw_shift;
-	    };
-	} else if (
-	    ((is_lineend_crlf || is_lineend_cr || is_lineend_thru)
+	if (is_zw_shift && (d2 != A_LF) && (d2 != A_CR)) {
+	    SKFputc(A_SP);
+	} else {
+	    SKFputc(ZW_ANN); SKFputc(ZW_ENT); /* dirty but needed  */
+	    SKFputc(A_SP);
+	    set_zw_shift;
+	};
+	if (((is_lineend_crlf || is_lineend_cr || is_lineend_thru)
 		&& (d2 == A_CR)) ||
 	    ((is_lineend_lf || (is_lineend_thru && (!detect_cr)))
 		&& (d2 == A_LF))) {
-	    if (is_zw_shift) {
-	    	SKFputc(ZW_LF); res_zw_shift;
-	    } else;
+	    SKFputc(ZW_LF); res_zw_shift;
 	    if (d2 == A_CR) set_detect_cr;
 	    if (d2 == A_LF) set_detect_lf;
 	} else ;
@@ -1871,7 +1831,7 @@ const char *x;
 	else if (is_euc(conv_cap)) SKFEUCSTROUT(x);
 	else if (out_bg(conv_cap)) SKFBGSTROUT(x);
 	else if (is_ucs_ufam(conv_cap)) SKFUNISTROUT(x);
-	else if (is_brgt(conv_cap)) SKFBRGTSTROUT(x);
+	else if (is_ucs_brgt(conv_cap)) SKFBRGTSTROUT(x);
 	else if (is_keis(conv_cap)) SKFKEISSTROUT(x); 
 }
 
@@ -1886,7 +1846,7 @@ void SKF1FLSH()
 	} else if (is_euc(conv_cap)) { SKFEUC1FLSH();
 	} else if (out_bg(conv_cap)) { SKFBG1FLSH();
 	} else if (is_ucs_ufam(conv_cap)) { SKFUNI1FLSH();
-	} else if (is_brgt(conv_cap)) { SKFBRGT1FLSH();
+	} else if (is_ucs_brgt(conv_cap)) { SKFBRGT1FLSH();
 	} else if (is_keis(conv_cap)) { SKFKEIS1FLSH(); 
 	} else;
 }
@@ -1964,15 +1924,13 @@ void SKFCRLF()
 	if (is_lineend_lf) fprintf(stderr,"L");
     };
 #endif
-    if (is_keis(conv_cap)) {
-    	ox_ascii_conv(A_NEL);
-    } else if (is_lineend_thru) {
-	if (first_detect_cr && detect_cr) {
+    if (is_lineend_thru) {
+	if (use_first_detect_cr && use_detect_cr) {
 	    ox_ascii_conv(A_CR);
-	    if (detect_lf) ox_ascii_conv(A_LF);
+	    if (use_detect_lf) ox_ascii_conv(A_LF);
 	} else {
-	    if (detect_lf) ox_ascii_conv(A_LF);
-	    if (detect_cr || !detect_lf) ox_ascii_conv(A_CR);
+	    if (use_detect_lf) ox_ascii_conv(A_LF);
+	    if (use_detect_cr || !use_detect_lf) ox_ascii_conv(A_CR);
 	};
     } else {
 	if (is_lineend_crlf || is_lineend_cr) ox_ascii_conv(A_CR);
@@ -2026,38 +1984,29 @@ unsigned short uu;
     int i;
     skf_ucode uukk;
     unsigned short uujj;
-    int	offset_p = 0;
+    int	offset_p = -1;
 
 #ifdef SKFDEBUG
     if (is_vvv_debug) {
-    	fprintf(stderr,"#decm: %x ",uu); fflush(stderr);
+    	fprintf(stderr,"#decm: %x ",uu);
+	(void) fflush(stderr);
     };
 #endif
     uu--; 	/* Note: uu has +1 bias.	*/
-    if (uu >= nkdc_lowsize) {	/* nkdc table overflow check */
-    	out_undefined(0,SKF_ENC_ERR);
-	return;
-    } else;
  /* limit = 32. In Unicode 5.0, longest word has 18 char decomposition */
     for (i=0;i<32 && ((uukk = nkdc_lowmap[uu]) != 0);i++, uu++) {
     	if ((uukk >= 0xa0) && (uukk < 0x3400)) {
 		offset_p = UNI_LAT_OFF;
-    	} else if ((uukk >= 0xf900) && (uukk < 0x10000)) {
-		offset_p = UNI_UP_OFF - 0xf900;
-    	} else if ((uukk >= 0x1d100) && (uukk < 0x1d1c1)) {
-		offset_p = UNI_COM_OFF - 0x1d100;
-    	} else if ((uukk >= 0x11090) && (uukk < 0x11600)) {
-		offset_p = UNI_KAITHI_OFF - 0x11090;
-    	} else if ((uukk >= 0x1ee00) && (uukk < 0x1ef00)) {
-		offset_p = UNI_ARAM_OFF - 0x1ee00;
-    	} else if ((uukk >= 0x1f100) && (uukk < 0x1f400)) {
-		offset_p = UNI_ECSP_OFF - 0x1f100;
+    	} else if ((uukk >= 0xf800) && (uukk < 0x10000)) {
+		offset_p = UNI_UP_OFF;
+    	} else if ((uukk >= 0x1d15e) && (uukk < 0x1d1c1)) {
+		offset_p = UNI_COM_OFF;
     	} else if ((uukk >= 0x2f801) && (uukk < 0x2fa1d)) {
-		offset_p = UNI_CJKC_OFF - 0x2f800;
+		offset_p = UNI_CJKC_OFF;
 	} else ;
 	
-	if ((offset_p != 0) 
-		&& ((uujj = nkdc_lowptr[uukk+offset_p]) != 0)) {
+	if ((offset_p >= 0) 
+		&& ((uujj = nkdc_lowptr[uukk-offset_p]) != 0)) {
 	    if ((decompose_recursion_depth++) > 32) {
 	    	/* Something in table is REALLY WRONG	*/
 		skferr(SKF_DECOMPERR,(long)uu,0);
@@ -2067,6 +2016,9 @@ unsigned short uu;
 	    decompose_buf[decompose_bufp++] = uukk;
 	};
     };
+#ifdef FOLD_SUPPORT
+    fold_count = 0;
+#endif
 }
 
 /* --------------------------------------------------------------- */
@@ -2182,18 +2134,3 @@ int	oank;
 /* --------------------------------------------------------------- */
 #endif /* UNI_ENCOMPOSE */
 #endif	/* UNI_DECOMPOSE */
-/* --------------------------------------------------------------- */
-void GBKR_finish_procedure()
-{
-    oconv_flush();
-    if (is_zW(conv_cap)) {
-	if (is_zw_shift) {
-	    SKFputc(ZW_LF); res_zw_shift;
-	} else;
-    } else if (is_hz(conv_cap)) {
-	if (is_hz_shift) {
-	    SKFputc(HZ_ANN); SKFputc(HZ_END);
-	};
-    } else;
-}
-/* --------------------------------------------------------------- */
