@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 1998-2011 Seiji Kaneko. All rights reserved.
+** Copyright (c) 1998-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     oconv.h:	Code conversion table related definitions
-    $Id: oconv.h,v 1.110 2012/03/17 08:22:12 seiji Exp seiji $
+    $Id: oconv.h,v 1.112 2015/04/26 15:08:38 seiji Exp seiji $
 **/
 #ifndef	OCONV_H_DEFINED
 #define OCONV_H_DEFINED
@@ -89,12 +89,16 @@ extern skf_ucode pref_subst_char;
 #define UPKANA_TBL_END	0x0001bfff
 #define NOTE_TBL_START	0x0001d000
 #define NOTE_TBL_END	0x0001ffff
+#define FITZPAT_TBL_START	0x0001f3fb
+#define FITZPAT_TBL_END	0x0001f3ff
 #define CJK_B_TBL_START	0x00020000	/* CJK UI Extension B/C/D */
 #define CJK_B_TBL_END	0x0002bfff
 #define CJK_C_TBL_START	0x0002f800	/* CJK UI compat suppl	  */
 #define CJK_C_TBL_END	0x0002faff
 #define TAG_TBL_START	0x000e0000
 #define TAG_TBL_END	0x000e007f
+#define SUP_PRIV_START	0x000f0000
+#define SUP_PRIV_END	0x0010ffff
 #define VARSEL_SUP_TBL_START 0x000e0100
 #define VARSEL_SUP_TBL_END 0x000e01ef
 #define UNI_HIGH_LIMIT	0x0010ffff
@@ -128,8 +132,7 @@ extern skf_ucode pref_subst_char;
 #define		OTBL_MONO	0x1000000UL /* Single byte type	   */
 #define		OTBL_JOHAB	0x2000000UL /* Johab		   */
 #define		OTBL_EBC	0x4000000UL /* EBCDIC Based	   */
-#define		OTBL_JIS	0x40000000UL /* JIS Kanji & Others */
-#define		OTBL_JISK	0x20000000UL /* JIS Kanji	   */
+#define		OTBL_JIS	0x40000000UL /* JIS Kanji	   */
 
 /* output table format:						   */
 /* Plane packing notation -
@@ -721,12 +724,7 @@ extern void	show_lang_tag ();
 
 extern void	SKF_STRPUT P_((unsigned short *));
 extern void	utf8_uriout P_((skf_ucode));
-#if 0
 extern char	*utf8_urioutstr P_((skf_ucode));
-#endif
-
-extern int	hex_conv_table[];
-
 /* ---- language tag controls ----- */
 extern void	output_language_tag_trigger P_(());
 /* ---- to make code small -------- */
@@ -780,6 +778,14 @@ extern int	get_combine_strength P_((skf_ucode));
 #define ENCOMPOSE_UPLIMIT	0x20000
 #define ENCOMPOSE_MASK_LEN	512	/* ENCOMPOSE_UPLIMIT/256 */		
 
+#if 0
+/* encompose: encompose is always enabled because of kana */
+struct encompose_ctbl {
+    skf_ucode c_identity;	/* candidate char		*/
+    int		leaf_id;	/* next leaf			*/
+};
+#endif
+
 struct encompose_leaf {
     skf_ucode	c_identity;	/* current char			*/
     skf_ucode	c_compose;	/* composed character		*/
@@ -806,11 +812,22 @@ extern int 	*encompose_list;
 #define is_weak_oidasi(x) (x & WEAK_OIDASI)
 #define is_strong_burasage(x) (x & STRONG_BURASAGE)
 
-#if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) && defined(NKF_COMPATMODE)
+#if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) 
 
 extern void SKF_rotoconv P_((int,int));
 extern void SKFROTTHRU P_((int,int));
 
 #endif
 
+
+/* -------------------------------------------------------------- */
+/* locale default codeset definition				  */
+/* -------------------------------------------------------------- */
+
+struct locale_codeset_name  {
+    unsigned long	langnat;	/* language and nation	  */
+    int		codename;
+};
+
+/* -------------------------------------------------------------- */
 #endif
