@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 2008-2011 Seiji Kaneko. All rights reserved.
+** Copyright (c) 2008-2015 Seiji Kaneko. All rights reserved.
 ** Copyright (c) 2003 The Internet Society. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
@@ -23,7 +23,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     out_encoder.c	output-side converter encode routines
-    $Id: out_encoder.c,v 1.22 2013/12/19 15:06:35 seiji Exp seiji $
+    $Id: out_encoder.c,v 1.25 2015/04/26 15:08:38 seiji Exp seiji $
 
     Punycode encode part is copyrighted by The Internet society.
 */
@@ -319,7 +319,6 @@ int oo_codeset;
     char	*desc;
 
     desc = i_codeset[oo_codeset].cname;
-    if (alt_desc != NULL) desc = (char *) alt_desc;
 
     if ((oo_codeset == codeset_utf16le) 
      || (oo_codeset == codeset_utf16be)
@@ -341,24 +340,15 @@ int oo_codeset;
 	} else {
 	    desc = "utf-32le"; 	
 	};
-    } else if (is_nkf_compat) {
-    	/* Note: I know THIS IS WRONG, but it's what nkf is doing  */
-    	if (oo_codeset == codeset_cp932) {
-	    desc = "shift_JIS";
-	} else if (oo_codeset == codeset_cp51932) {
-	    desc = "euc-jp";
-	} else if (oo_codeset == codeset_cp5022x) {
-	    desc = "iso-2022-jp";
-	} else;
     } else;
 
-    for (i=0; i< 32; i++, desc++) {
+    for (i=0; i< 16; i++, desc++) {
 	if (*desc == '\0') break;
 	SKFcputc(SKFtoupper(*desc));
     };
 }
 
-#if defined(FUTURESUPPORT)
+#if 0
 /* reserved for future use */
 /* --------------------------------------------------------------- */
 int is_idnspace(ch)	/* rfc3454 compliant space detection	   */
@@ -532,7 +522,7 @@ void SKFrCRLF()
 /* --------------------------------------------------------------- */
 /* hex/octal encoder						   */
 /* --------------------------------------------------------------- */
-int hex_conv_table[16] = {
+static int hex_conv_table[16] = {
   '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
 };
 
@@ -735,6 +725,7 @@ skf_ucode ch;
 {
     int i;
     int	pres;
+    skf_ucode rch;
 
 #ifdef SKFPDEBUG
     if (is_vv_debug) {
@@ -744,30 +735,36 @@ skf_ucode ch;
     };
 #endif
     if (o_encode_stat == 0) {
-	if (is_puny_strdelim(ch) || (ch == '/') || (ch < 0)) {
+    	if (ch < 0) {
+	    return;
+	} else if (is_puny_strdelim(ch) || (ch == '/')) {
 		/* if basic, ch <= 0x80	 */
 		/* maybe some encoding violate pos 0x3f '/', but this */
 		/* won't be serious matter for punycode cutting. */
 	    while (!enc_pre_qempty()) SKFcputc(enc_pre_deque());
-	    if (ch >= 0) SKFcputc(ch);
+	    SKFcputc(ch);
 	    enc_pre_qreset();
-	} else if (!is_puny_basic(ch)) {
+	} else if (!is_puny_basic(ch) || (ch == A_ESC)) {
 	    enc_pre_enque(ch);
 	    o_encode_stat = TRUE;
-	    puny_maxcp = 1;
 	} else {
 	    enc_pre_enque(ch);
 	    o_encode_stat = TRUE;
 	};
     } else {
-	if ((is_puny_strdelim(ch)) || enc_pre_qfull() || (ch < 0)) {
+	if ((is_puny_strdelim(ch)) || enc_pre_qfull()) {
+	    rch = ch;
+	    if (ch < 0) {
+	    	ch = 0;
+	    };
+	    enc_pre_enque(ch);
 	    encode_post_wptr = O_PSTQDEPTH;
 	    if (puny_maxcp == 0) {
 	    	while (!enc_pre_qempty()) {
 		    SKFcputc(enc_pre_deque());
 		};
 	    } else {
-		pres = punycode_encode((enc_pre_q_w_ptr),
+		pres = punycode_encode((enc_pre_q_w_ptr - 1),
 				 encode_pre_queue,
 				 &encode_post_wptr,
 				 encode_post_queue);
@@ -777,16 +774,12 @@ skf_ucode ch;
 		    SKFcputc('-'); SKFcputc('-');
 		    for (i=0;i<encode_post_wptr;i++)
 			SKFcputc(encode_post_queue[i]);
-		} else;
+		};
 	    };
 	    o_encode_stat = FALSE;
 	    enc_pre_qreset();
 	    puny_maxcp = 0;
-	    if (is_puny_strdelim(ch)) SKFrputc(ch);
-	    else if (ch >= 0) {
-	    	enc_pre_enque(ch);
-		o_encode_stat = TRUE;
-	    } else;
+	    if (is_puny_delim(rch)) SKFcputc(rch);
 	} else {
 	    if (!is_puny_basic(ch) || (ch == A_ESC)) puny_maxcp = 1;
 	    enc_pre_enque(ch);
@@ -1029,31 +1022,28 @@ int	ch;
 	else if (ch == sOCD) fprintf(stderr,"sOCD");
 	else if (ch == sKAN) fprintf(stderr,"sKAN");
 	else if (ch == sUNI) fprintf(stderr,"sUNI");
+	else if (ch == sSUSP) fprintf(stderr,"sSUSP");
 	else if (ch == sFLSH) fprintf(stderr,"sFLSH");
 	else if (ch == mFLSH) fprintf(stderr,"mFLSH");
 	else fprintf(stderr,"%x",ch);
 	fprintf(stderr,"(%d/%d-%d)",o_encode_lm,o_encode_lc,
 		prequeue_depth);
+	if (ch >= 0) {
+	    fprintf(stderr,"%c%c%c%c:",
+		(no_early_mime_out(nkf_compat) ? 'N':'E'),
+		(table_mime_tail(ch) ? 'T':'C'),
+		(mime_safe(ch) ? 't':'m'),
+		(is_white(ch) ? 'w':'d'));
+	} else;
     };
 #endif
 
-#ifdef SKFDEBUG
-    if (is_vv_debug && (ch >= 0)) {
-	fprintf(stderr,"%c%c%c%c:",
-	    (no_early_mime_out(nkf_compat) ? 'N':'E'),
-	    (table_mime_tail(ch) ? 'T':'C'),
-	    (mime_safe(ch) ? 't':'m'),
-	    (is_white(ch) ? 'w':'d'));
-    } else;
-#endif
-
     if (ch == sOCD) {   /* should not be here, but throw anyway.   */
        pre_ch = ch; return;
-    };
-    if (o_encode_stat == 0) {	/* under testing		   */
-		/* b64, hex, dec and perc is o_encode_stat == TRUE */
-		/* Puny never comes here, So mimeb, mimeq and	   */
-		/* rfc2231(not supported) does arrive here.	   */
+    } else if (o_encode_stat == 0) {	/* under testing	   */
+	/* b64, hex, dec and perc is always o_encode_stat == TRUE  */
+	/* Puny never comes here, So mimeb, mimeq and		   */
+	/* rfc2231(not supported) arrive here.			   */
 	if (ch < 0) {
 	    queue_to_mime(o_encode);
 	    pre_ch = ch;
@@ -1096,9 +1086,9 @@ int	ch;
 	    base64_enc(ch,o_encode);
 	    pre_ch = ch;
 	    return;
+		/* BASE64 OR MIME-B OR MIME-Q OR RFC2231 */
 	} else if (is_o_encode_b64(o_encode) ||
-			is_o_encode_2231(o_encode) ||
-			is_o_encode_mimeb(o_encode)) {
+			is_o_encode_mimes(o_encode)) {
 	    if (o_encode_pend > 0) {  /* sweep out pending chars   */
 	    	if (ch == mFLSH) {	/* JIS case		   */
 			/* encode should sweep esc seq. beforehand */
@@ -1110,11 +1100,21 @@ int	ch;
 		    queue_to_mime(o_encode);
 		};
 	    } else;
+#if 0
+	    if ((ch == sFLSH) || (ch == mFLSH) || (ch == sSUSP)) {
+#else
 	    if ((ch == sFLSH) || (ch == mFLSH)) {
+#endif
 		    /* puny length limit is considered in other */
 		    /* routine. Here we test only MIMEs	   */
 		queue_to_mime(o_encode);
 		return;
+	    } else if (ch == sSUSP) {
+	    	queue_to_mime(o_encode);
+		if (o_encode_stat > 0) {
+			mime_tail_gen(o_encode);
+		} else;
+		return;
 	    } else {
 		base64_enc(sOCD,o_encode);/* sweep remains in buffer */
 	    };
@@ -1130,35 +1130,8 @@ int	ch;
 #endif
 	    o_encode_pend = 0;
 	    return;
-	} else if (is_o_encode_mimeq(o_encode)) {
-	    if (o_encode_pend > 0) {  /* sweep out pending chars   */
-	    	if (ch == mFLSH) {	/* JIS case		   */
-			/* encode should sweep esc seq. beforehand */
-		    queue_to_mime(o_encode);
-		    o_encode_pend = 0;
-		    return;
-		} else {
-		    mime_tail_gen(o_encode);
-		    queue_to_mime(o_encode);
-		};
-	    } else;
-	    if ((ch == sFLSH) || (ch == mFLSH)) {
-		    /* puny length limit is considered in other */
-		    /* routine. Here we test only MIMEs	   */
-		queue_to_mime(o_encode);
-		return;
-	    } else;
-	    mime_tail_gen(o_encode);
-	    if (is_true_lineend(ch)) {
-	    	SKFrCRLF();
-	    };
-	    o_encode_stat = 0;	/* terminate encodings.		   */
-#ifdef SUPERFOLD
-	    o_encode_lc_save = 0;
-#endif
-	    return;
 	} else if (is_o_encode_oct(o_encode)) {
-	    if (!is_lineend(ch) && (ch >= 0)) {
+	    if (!is_lineend(ch)) {
 		SKFcputc(BSL_DEL);
 		char2oct(ch);
 		return;
@@ -1173,14 +1146,14 @@ int	ch;
     } else if ((table_mime_tail(ch) || is_white(ch)) 
 		&& !no_early_mime_out(nkf_compat)
 		&& (!is_kanji_shift && !is_ucs_utf16(conv_cap))
-		&& is_o_encode_mime(o_encode)) {
+		&& is_o_encode_mimebq(o_encode)) {
 		/* Note: sFLSH and sEOF is also handled here	   */
 	o_encode_pend = 1;
 	enc_pre_enque(ch);
 #endif
     } else if ((o_encode_pend > 0)
 		&& !no_early_mime_out(nkf_compat)
-		&& is_o_encode_mime(o_encode)) {
+		&& is_o_encode_mimebq(o_encode)) {
 	if (!mime_safe(ch)) {
 	    if ((o_encode_pend == 2) && (o_encode_stat > 0)) {
 	    	mime_tail_gen(o_encode);
@@ -1225,26 +1198,16 @@ void encoder_tail()
     } else;
 #endif
     if (o_encode_stat == 0) {	/* not in mime'd		   */
-	if ((is_o_encode_mimeq(o_encode)) 
-	   || (is_o_encode_2231(o_encode))
-	   || (is_o_encode_mimeb(o_encode))) {
+	if (is_o_encode_mimes(o_encode)) {
 	    o_encode_lc = 0;
 	    o_encode_lm = 0;
-	} else if (is_o_encode_oct(o_encode) 
-		|| is_o_encode_uri(o_encode)
-		|| is_o_encode_hex(o_encode)
-		|| is_o_encode_perc(o_encode)
-		|| is_o_encode_q(o_encode)) {
-		;
 #ifndef SWIG_EXT
 	} else if (is_o_encode_b64(o_encode)) {
 	    SKFrCRLF();
 #endif
 	} else;
     } else {			/* in mime			   */
-	if ((is_o_encode_mimeq(o_encode)) 
-	   || (is_o_encode_2231(o_encode))
-	   || (is_o_encode_mimeb(o_encode))) {
+	if (is_o_encode_mimes(o_encode)) {
 	    mime_tail_gen(o_encode);
 	    o_encode_lc = 0;
 	    o_encode_lm = 0;
@@ -1282,34 +1245,3 @@ skf_ucode ch;
     o_encode_lm = 0;
 }
 /* --------------------------------------------------------------- */
-void puny_finish_procedure()
-{
-    int i;
-    int	pres;
-
-    oconv_flush();
-    if ((o_encode_stat == 0) || (puny_maxcp == 0)) {
-	while (!enc_pre_qempty()) SKFcputc(enc_pre_deque());
-	o_encode_stat = FALSE;
-	enc_pre_qreset();
-	puny_maxcp = 0;
-    } else {
-    /* Note: strdelim includes range 0x00 - 0x20 */
-	encode_post_wptr = O_PSTQDEPTH;
-	pres = punycode_encode((enc_pre_q_w_ptr),
-			 encode_pre_queue,
-			 &encode_post_wptr,
-			 encode_post_queue);
-	if (pres == 0) {
-	    SKFcputc(PUNY_PRFX1);
-	    SKFcputc(PUNY_PRFX2);
-	    SKFcputc('-'); SKFcputc('-');
-	    for (i=0;i<encode_post_wptr;i++)
-		SKFcputc(encode_post_queue[i]);
-	} else;
-	o_encode_stat = FALSE;
-	enc_pre_qreset();
-	puny_maxcp = 0;
-    };
-}
-/* --------------------------------------------------------------- */
