@@ -23,7 +23,7 @@
 **********************************************************************
 ** plug_convert.c:	various converter routines for skf
 **		to treat something dirty ;-)
-** $Id: plug_convert.c,v 1.114 2013/07/22 13:42:44 seiji Exp seiji $
+** $Id: plug_convert.c,v 1.116 2014/12/26 16:32:07 seiji Exp seiji $
 **	x0201conv:	x0201 kana converter
 **	new2oldJis:	x0208(1983,1990) to C6226(1978)
 **	lig_x0213_out:	X0213/ISCII ligature out 
@@ -263,13 +263,9 @@ skf_ucode ch;
 	    c2 = uni_lig_x0213_24[c1];	/* 0x24xx, 0x25xx	   */
 	    if (c2 != 0) {
 		post_oconv(c2);
-#if 1
-#ifdef NKF_COMPATMODE
 		if (!is_ms_213c(conv_cap) && !is_euc_213c(conv_cap)
 		    	 && !is_jis_213c(conv_cap) 
-#ifdef NKF_COMPATMODE
 			 && !is_nkf_compat
-#endif
 		     && (in_codeset != codeset_x0213)
 		     && (in_codeset != codeset_x213a)
 		     && (in_codeset != codeset_x0213_s)
@@ -282,12 +278,6 @@ skf_ucode ch;
 		} else {
 		    post_oconv(0x309a);
 		};
-#else
-		post_oconv(0x309a);
-#endif
-#else
-		post_oconv(0x309a);
-#endif
 	    } else {
 		out_undefined(ch,SKF_IOUTUNI);
 	    };
@@ -392,6 +382,9 @@ skf_ucode ch;
 #endif
 	};
 #endif
+    } else if (ch < 0xd9d0) {	/* mosaics (0xd900 - 0xd9df)	   */
+    	/* do anything before this point.			   */
+	out_undefined(ch,SKF_IOUTUNI);
     } else {
 	out_undefined(ch,SKF_IOUTUNI);
     };
@@ -602,9 +595,6 @@ skf_ucode c2;
 	post_oconv(0x29);
     } else if ((c1 >= 0xfc) && (c1 <= 0xfe)) { /* Enclosed kana */
 	CJK_circled(0x2ff4+c1,8);
-    } else if (c1 == 0xff) {	/* new era ligature		*/
-    	post_oconv(0x4ee4);
-    	post_oconv(0x548c);
     } else {		  /* just undefied 		   */
 	out_undefined(c2,SKF_OUNDEF);
     };
@@ -884,9 +874,9 @@ skf_ucode c1;
     	input:		c1 - input character
 	output:		none
 */
-static char *u_bra_lig_supl[11] = {
+static char *u_bra_lig_supl[10] = {
   "DJ","CL","COOL","FREE","ID","NEW","NG","OK",
-  "SOS","UP!","VS"
+  "UP!","VS"
 };
 
 void enc_alpha_supl_conv(c1)
@@ -1074,7 +1064,7 @@ void show_lang_tag()
 	    SKFputc((int)((ll >> 8) & 0x7f));
 	    SKFputc((int)(ll & 0x7f));
 	};
-    } else if (is_brgt(conv_cap)) { /* TAD Announcer	   */
+    } else if (is_ucs_brgt(conv_cap)) { /* TAD Announcer	   */
 	tron_announce();
     };
 }
@@ -1379,7 +1369,7 @@ int	c2,c1;
 /* JEF support: similar as KEIS, but code is completely different  */
 /* --------------------------------------------------------------- */
 
-int jef_conv(c2,c1)
+void jef_conv(c2,c1)
 int	c2,c1;
 {
     skf_ucode k0;
@@ -1388,7 +1378,7 @@ int	c2,c1;
 
     c1 &= 0x7f;
     if ((c1 <= 0x20) || (c1 == 0x7f) || (c2 < 0x43) || (c2 > 0x7f)) {
-	    in_sbroken(c2,c1); return(0);
+	    in_sbroken(c2,c1); return;
     };
 /* --- various kanji area ---------------------------------------- */
 /* 	supported X-0208 kanji only				   */
@@ -1398,7 +1388,7 @@ int	c2,c1;
     k0 = jef_t[c_loc];
     if (k0 != 0) oconv(k0);
     else in_sbroken(c2,c1);
-    return(0);
+    return;
 }
 
 /* --------------------------------------------------------------- */
@@ -1414,10 +1404,7 @@ skf_ucode c1;
 #endif
     if (use_latin2html) enbl = latin2html(c1);
     if (use_latin2tex && !enbl) enbl = latin2tex(c1);
-#ifdef NKF_COMPATMODE
-    if ((use_latin2null && is_nkf_compat)
-    	|| is_no_bfc) return;	/* discard 'em	   */
-#endif
+    if (use_latin2null || is_no_bfc) return;	/* discard 'em	   */
 
     if (!enbl) {
 	if ((c1 >= 0x3000) && (c1 <= 0x4e00) &&
@@ -1803,56 +1790,27 @@ static const char *uni_latin_html_26[16] = {  /* 0x2660 - 0x266f   */
     NULL, "&flat;", "&natur;", "&sharp;"
 };
 
-static int hex_conv_table_s[16] = {
-  '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'
-};
-
 int latin2html(c1)
 skf_ucode c1;
 {
     skf_ucode c2,c3;
-    skf_ucode ch;
     const char *sym = NULL;
 
 #ifdef SKFDEBUG
-    if (is_vv_debug)
-		fprintf(stderr," latin2html: %06lx",(unsigned long)c1);
+	if (is_vv_debug)
+		fprintf(stderr," latin2html: %04x",c1);
 #endif
-    c2 = (c1 >> 8) & 0xffff; c3 = (c1 & 0xff);
+    c2 = (c1 >> 8) & 0xff; c3 = (c1 & 0xff);
     if (((c1 < A_DEL) && (!use_htmlsanitize ||
 		(uni_latin_html_0[c1 - 0x20] == NULL))) 
 		|| (use_latin2htmld)) {
-	if (c1 >= UNI_HIGH_LIMIT) {
-	    out_undefined(c1,SKF_UNSURG);
-	    return(TRUE);
-	} else;
 	SKFSTROUT("&#");
-	ch = c1;
-	if (c1 >= 1000000) {
-	    SKFputc(hex_conv_table_s[(c1 / 1000000)]);
-	    c1 = c1 % 1000000;
-	} else;
-	if ((c1 >= 100000) || (ch >= 1000000)) {
-	    SKFputc(hex_conv_table_s[(c1 / 100000)]);
-	    c1 = c1 % 100000;
-	} else;
-	if ((c1 >= 10000) || (ch >= 100000)) {
-	    SKFputc(hex_conv_table_s[(c1 / 10000)]);
-	    c1 = c1 % 10000;
-	} else;
-	if ((c1 >= 1000) || (ch >= 10000)) {
-	    SKFputc(hex_conv_table_s[(c1 / 1000)]);
-	    c1 = c1 % 1000;
-	} else;
-	SKFputc(hex_conv_table_s[(c1 / 100)]);
-	c1 = c1 % 100;
-	SKFputc(hex_conv_table_s[(c1 / 10)]);
-	c1 = c1 % 10;
-	SKFputc(hex_conv_table_s[c1 & 0xfU]);
-	SKFputc(';');
+	printf("%03d;",c1); 	/* output 3 digit at least	   */
 	return(TRUE);
     } else if (use_latin2htmlh) {
-    	;
+	SKFSTROUT("&#x");
+	printf("%03x;",c1); 	/* output 3 digit at least	   */
+	return(TRUE);
     } else if (use_latin2htmlu) {
 	utf8_uriout(c1);
 	return(TRUE);
@@ -1869,6 +1827,14 @@ skf_ucode c1;
 	};
     } else if (c2 == 0x02) {
 	switch (c3) {
+#if 0
+	    case 0x26: sym ="&Adot;"; break;
+	    case 0x27: sym ="&adot;"; break;
+	    case 0x2e: sym ="&Odot;"; break;
+	    case 0x2f: sym ="&odot;"; break;
+	    case 0x32: sym ="&Ymacr;"; break;
+	    case 0x33: sym ="&ymacr;"; break;
+#endif
 	    case 0xbc: sym ="&apos;"; break;
 	    case 0xc6: sym ="&circ;"; break;
 	    case 0xc7: sym ="&caron;"; break;
@@ -1928,20 +1894,9 @@ skf_ucode c1;
 
     if (sym != NULL) {
 	SKFSTROUT(sym);
-    } else {	/* not dec or hex. go hex anyway		  */
-	SKFSTROUT("&#x");
-	if (c1 >= 0x10000) {
-	    SKFputc(hex_conv_table_s[(c1 >> 20) & 0x0fU]);
-	    SKFputc(hex_conv_table_s[(c1 >> 16) & 0x0fU]);
-	    c1 &= 0xffffU;
-	};
-	if (c1 >= 0x100) {
-	    SKFputc(hex_conv_table_s[(c1 >> 12) & 0x0fU]);
-	};
-	SKFputc(hex_conv_table_s[(c1 >> 8) & 0x0fU]);
-	SKFputc(hex_conv_table_s[(c1 >> 4) & 0x0fU]);
-	SKFputc(hex_conv_table_s[(c1 & 0x0fU)]);
-	SKFputc(';');
+    } else {
+	SKFSTROUT("&#");
+	printf("%03d;",c1); 	/* output 3 digit at least	   */
     };
 
     return(TRUE);
@@ -2221,6 +2176,7 @@ static const char *uni_latin_tex_26[16] = {  /* 0x2660 - 0x266f	   */
     NULL, "\\[\\flat\\]", "\\[\\natural\\]", "\\[\\sharp\\]"
 };
 
+/*@-branchstate@*/
 int latin2tex(c1)
 skf_ucode c1;
 {
@@ -2232,11 +2188,10 @@ skf_ucode c1;
 	if (is_vv_debug)
 		fprintf(stderr," latin2tex: %04x",c1);
 #endif
-    c2 = (c1 >> 8) & 0xffff; c3 = (c1 & 0xff);
+    c2 = (c1 >> 8) & 0xff; c3 = (c1 & 0xff);
     if (c2 == 0) {
 	if ((c3 == 0x5c) && !use_htmlsanitize) {
-	    SKFSTROUT("\\\\");
-	    out_enbl = TRUE;
+	    sym = "\\\\";
 	} else if (c3 < A_DEL) { 
 	    post_oconv(c3); out_enbl = TRUE;
 	} else if ((c3 >= 0xa0) && (c3 <= 0xff)) {
@@ -2244,7 +2199,6 @@ skf_ucode c1;
 	} else {
 	    out_enbl = FALSE; 
 	};
-	if (sym == NULL) out_enbl = FALSE; 
     } else if (c2 == 0x01) {
 /* This section regards ogonek as cedilla, but it is doubtful. */
 	if (c3 <= 0x7f) {
@@ -2340,6 +2294,7 @@ skf_ucode c1;
     if (sym != NULL) SKFSTROUT(sym);
     return(out_enbl);
 }
+/*@+branchstate@*/
 
 /* --------------------------------------------------------------- */
 static const unsigned short viqr_map[256] = {
