@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 1997-2009 Seiji Kaneko. All rights reserved.
+** Copyright (c) 1997-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
     preconvert.c	code detector and converter
-    $Id: preconvert.c,v 1.117 2013/12/19 15:11:08 seiji Exp seiji $
+    $Id: preconvert.c,v 1.121 2015/04/26 15:08:38 seiji Exp seiji $
 */
 
 #include <stdio.h>
@@ -36,9 +36,12 @@
 #ifdef SKFDEBUG
 #define set_rescase(x)	rescase = x
 #define reset_rescase() rescase = 0
+#define add_rescase() rescase++
 #else
 #define set_rescase(x)	;
 #define reset_rescase() ;
+#define add_rescase() ;
+#define add_rescase() ;
 #define debug_fault(x,y)  ;
 #endif
 
@@ -51,7 +54,7 @@
 #define	is_det_utf7 is_ucs_utf7(i_codeset[in_codeset].encode)
 #define	is_det_keis is_keis(i_codeset[in_codeset].encode)
 #define	is_det_latin is_jis(i_codeset[in_codeset].encode)
-#define	is_det_brgt is_brgt(i_codeset[in_codeset].encode)
+#define	is_det_brgt is_ucs_brgt(i_codeset[in_codeset].encode)
 #define	is_det_transp is_ucs_transp(i_codeset[in_codeset].encode)
 
 #define is_code_undefine (in_codeset < 0) 
@@ -155,7 +158,7 @@
 /* --------------------------------------------------------------- */
 void	dump_name_of_code P_((int, int, int));
 static int single_possible P_((int,int,int,int,int,int,int,int,
-	    int,int,int,int));
+	    int,int,int,int,int,int));
 static int	eval_encoding P_((long,long));
 static long	is_valid_utf8_seq P_((int,int,int,int));
 static void	set_le_parse P_((int,int,int));
@@ -197,8 +200,8 @@ static int viscii_v[32] = {
 /* SCOREBOARDING detecter					   */
 /* --------------------------------------------------------------- */
 static int 
-single_possible(jis,sjis,euc,utf8,utf7,ucs2,keis,latin,b5,gb,kr,hz)
-int jis,sjis,euc,utf8,utf7,ucs2,keis,latin,b5,gb,kr,hz;
+single_possible(jis,sjis,euc,utf8,utf7,ucs2,keis,latin,b5,gb,kr,hz,jl,sl)
+int jis,sjis,euc,utf8,utf7,ucs2,keis,latin,b5,gb,kr,hz,jl,sl;
 {
     if (sjis && euc && utf8 && utf7 && ucs2 && keis && latin
 	&& b5 && gb && kr && hz ) {
@@ -228,7 +231,7 @@ int jis,sjis,euc,utf8,utf7,ucs2,keis,latin,b5,gb,kr,hz;
     } else if (jis && sjis && euc && utf8 && utf7 && keis && latin
 	&& b5 && gb && kr && hz ) {
 	set_rescase(305);
-	in_codeset = codeset_utf16le;
+	in_codeset = codeset_utf16lenb;
 	return(1);
 #ifdef KEIS_DETECT
     } else if (jis && sjis && euc && utf8 && utf7 && ucs2 && latin
@@ -268,7 +271,8 @@ int jis,sjis,euc,utf8,utf7,ucs2,keis,latin,b5,gb,kr,hz;
 	&& b5 && gb && kr && hz &&
 	skf_is_ja(skf_input_lang)) {	/* JIS or SJIS -> SJIS	  */
 	set_rescase(312);
-	in_codeset = codeset_sjis;
+	if (jl > sl) in_codeset = codeset_jis;
+	else in_codeset = codeset_sjis;
 	return(1);
 #endif
     } else;
@@ -583,14 +587,14 @@ int preConvert (f)
    /* detect encoding status */
     int		utf7fault = 0, utf7likely = 0, utf7unlikely = 0;
     int		utf8fault = 0, utf8likely = 0, utf8unlikely = 0;
-#ifdef SWIG_EXT
-    int		ucs2fault = 1, ucs2likely = 0, ucs2unlikely = 1;
-#else
+#if !defined(SWIG_EXT) || (defined(SWIGPYTHON) && defined(SKF_PYTHON3)) || defined(SWIGRUBY) || defined(HAVE_FAST_LWLSTRLEN)
     int		ucs2fault = 0, ucs2likely = 0, ucs2unlikely = 0;
+#else
+    int		ucs2fault = 1, ucs2likely = 0, ucs2unlikely = 1;
 #endif
     int		sjisfault = 0, sjislikely = 0, sjisunlikely = 0;
     int		jisfault = 0, jislikely = 0, jisunlikely = 0;
-    int		keisfault = 0, keislikely = 0, keisunlikely = 0;
+    int		keisfault = 0, keislikely = 0, keisunlikely = 2;
     int		latinfault = 0, latinlikely = 0, latinunlikely = 0;
     int		b5fault = 0, b5likely = 0, b5unlikely = 0;
 		/* BIG5 (ETen and cp950) */
@@ -639,6 +643,9 @@ int preConvert (f)
 
 /* -- line delimiter character detections ------------------------ */
     long	ic_count = 0;
+#if !defined(SWIG_EXT) || (defined(SWIGPYTHON) && defined(SKF_PYTHON3)) || defined(SWIGRUBY) || defined(HAVE_FAST_LWLSTRLEN)
+    int		icw,icc;
+#endif
     int		le_u_cr_detect = 0;
     int		le_n_cr_detect = 0;
     int		le_u_lf_detect = 0;
@@ -701,7 +708,6 @@ int preConvert (f)
     if (kuni_opt) {	/* limit detection to JIS, EUC and SJIS	   */
 	utf8fault = TRUE; utf7fault = TRUE; keisfault = TRUE;
 	latinfault = TRUE; det_limit = 2; lk_power = 1;
-	b5fault++;
     } else if (fuzzy_detect) {
 	lk_power = 2;
 	det_limit = DET_LIMIT * lk_power;
@@ -782,7 +788,7 @@ int preConvert (f)
 	if ((sjislikely > (SJIS_LK_LIMIT * lk_power)) && !sjisfault) { 
 	    in_codeset = codeset_sjis; debug_fault("sjis",105); };
 	if ((ucs2likely > (UCS2_LK_LIMIT * lk_power)) && !ucs2fault) { 
-	    in_codeset = codeset_utf16le; debug_fault("ucs2",106); };
+	    in_codeset = codeset_utf16lenb; debug_fault("ucs2",106); };
 	if ((jislikely > (JIS_LK_LIMIT * lk_power)) && !jisfault) { 
 	    in_codeset = codeset_jis; debug_fault("jis",104); };
 	if ((latinlikely > (LATIN_LK_LIMIT * lk_power)) && !latinfault) { 
@@ -819,7 +825,8 @@ int preConvert (f)
 	};
 	(void)single_possible(jisfault,sjisfault,eucfault,utf8fault,
 		utf7fault,ucs2fault,keisfault,latinfault,b5fault,
-		gbfault,krfault,hzfault);
+		gbfault,krfault,hzfault,
+		jislikely-jisunlikely,sjislikely-sjisunlikely);
 
 	/* check again */
 	if ((!is_code_undefine) && (!is_lt_undefine || !lf_should_test)) 
@@ -859,9 +866,11 @@ int preConvert (f)
 /* ---------------------------------------------------- */
 /* code preparation					*/
 /* ---------------------------------------------------- */
+#if 0
 	if (c1 == A_BS) {	/* groff output hack		   */
 	    enque(c1); in_var_unshift; continue;
 	};
+#endif
 	rc1 = c1;
 	/* if euc7 shifted, set bit8 (right plane)		   */
 	if ((e7upper == 1) && (c1 > A_SP) && (c1 < A_DEL)) { 
@@ -890,7 +899,7 @@ int preConvert (f)
 	    if (ucs2ccnt != 0) { ucs2fault++; };
 	    if (vsqccnt != 0) { vsqfault++; };
 	    if (sjisccnt != 0) { sjisfault++; };
-#if KEIS_DETECT
+#ifdef KEIS_DETECT
 	    if (keisccnt != 0) { keisfault++; };
 #endif
 	    if ((rc1 == sEOF) || is_lineend(c1)) enque(rc1); 
@@ -906,96 +915,181 @@ int preConvert (f)
 /* assumed no Gurmurki, Gujarati, Malayalam appears.	*/
 /* ---------------------------------------------------- */
 	if (le_detect == 0) {
-#ifndef SWIG_EXT
-	  if (is_code_undefine
-		|| is_ucs_utf16(i_codeset[in_codeset].encode)) {
+#if !defined(SWIG_EXT) || (defined(SWIGPYTHON) && defined(SKF_PYTHON3)) || defined(SWIGRUBY) || defined(HAVE_FAST_LWLSTRLEN)
+	  if (!ucs2fault && (is_code_undefine
+		|| is_ucs_utf16(i_codeset[in_codeset].encode))) {
 /* ---------------------------------------------------- */
-/* ucs2							*/
+/* utf-16							*/
 /* ---------------------------------------------------- */
-	    if ((rc1 == A_CR) && (c2 == 0)
-		&& (in_big_endian || ((ic_count & 0x01) == 0))) {
-		 /* if we don't know code, or UCS2B		   */
-		le_u_cr_detect = TRUE;
-		     /* I know it is too aggressive, but UCS2 wo BOM   */
-		     /* is hard to determine anyway.		   */
-		if (!le_u_lf_detect) le_u_crf_detect = TRUE;
-		if (((c3 == A_CR) || (c3 == A_LF)) && (c4 == 0)) {
-		    set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
-		    in_codeset = codeset_utf16le; set_in_big_endian;
-		    enque(rc1);
-		    set_rescase(133); 
-		    if (is_code_undefine) continue;
-		    else break;
-		};
-	    } else if ((rc1 == 0) && (c2 == A_CR) 
-		    && (!in_big_endian || ((ic_count & 0x01) == 1))) {
-		le_u_cr_detect = TRUE; /* see comment above.	   */
-		if (!le_u_lf_detect) le_u_crf_detect = TRUE;
-		if (((c4 == A_CR) || (c4 == A_LF)) && (c3 == 0)) {
-		    set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
-		    in_codeset = codeset_utf16le; set_in_ltl_endian;
-		    enque(rc1);
-		    set_rescase(134);
-		    if (is_code_undefine) continue;
-		    else break;
-		};
-	    } else if ((rc1 == A_LF) && (c2 == 0)
-		    && (in_big_endian || ((ic_count & 0x01) == 0))) {
-		le_u_lf_detect = TRUE;
-		if (((c3 == A_CR) || (c3 == A_LF)) && (c4 == 0)) {
-		    set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
-		    in_codeset = codeset_utf16le; set_in_big_endian;
-		    enque(rc1);
-		    set_rescase(135); 
-		    if (is_code_undefine) continue;
-		    else break;
-		};
-	    } else if ((rc1 == 0) && (c2 == A_LF)
-		    && (!in_big_endian || ((ic_count & 0x01) == 1))) {
-		le_u_lf_detect = TRUE;
-		if (((c4 == A_CR) || (c4 == A_LF)) && (c3 == 0)) {
-		    set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
-		    in_codeset = codeset_utf16le; set_in_ltl_endian;
-		    enque(rc1);
-		    set_rescase(136);
-		    if (is_code_undefine) continue;
-		    else break;
-		};
-	    } else {
-		if (((c3 == A_CR) || (c3 == A_LF)) && (c2 == 0)) {
-		    if (c3 == A_CR) le_u_cr_detect = TRUE;
-		    if (c3 == A_LF) le_u_lf_detect = TRUE;
-		    set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
-		    in_codeset = codeset_utf16le; set_in_ltl_endian;
-		    enque(rc1);
-		    set_rescase(137);
-		    if (is_code_undefine) continue;
-		    else break;
-		} else if (((c2 == A_CR) || (c2 == A_LF))
-				    && (c3 == 0)) {
-		    set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
-		    in_codeset = codeset_utf16le; set_in_big_endian;
-		    enque(rc1);
+	    icw = (ic_count & 0x03);
+	    icc = (ic_count & 0x01);
+	/* code detection by 0x00 */
+	    if (is_code_undefine) {
+		if ((rc1 == 0) && (c2 <= 0x01) && (icw == 3)) {
+			/* can be BOM */
+		    in_codeset = codeset_utf32le;
+		    set_in_ltl_endian;
+		    set_rescase(132); 
+		} else if ((rc1 <= 0x01) && (c2 == 0) && (icw == 1)) {
+			/* can be BOM */
+		    in_codeset = codeset_utf32be;
+		    set_in_big_endian;
+		    set_rescase(134); 
+		} else if ((rc1 == 0) && (c1 >= 0x04) && (icc == 1)) {
+			/* U-40000 or above not likely appear	*/
+		    in_codeset = codeset_utf16lenb; /* no BOM	*/
+		    set_in_ltl_endian;
+		    set_rescase(136); 
+		} else if ((rc1 >= 0x04) && (c1 == 0) && (icc == 1)) {
+		    in_codeset = codeset_utf16benb; /* no BOM	*/
+		    set_in_big_endian;
 		    set_rescase(138); 
-		    if (is_code_undefine) continue;
-		    else break;
-		} else;
-		set_le_parse(
-			le_u_cr_detect,le_u_lf_detect,le_u_crf_detect);
+		} else;		/* not confirmed		*/
+	    } else;		/* already know codes		*/
+
+	/* UTF16 and UTF32 lineend character detection		*/
+	    if (is_lt_undefine && lf_should_test) {
+	    	if ((rc1 == 0) && (c2 == 0) && 
+		    (c3 == A_CR) && (c4 == 0) &&
+			(!in_big_endian || (icw == 3))) {
+		    set_in_ltl_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_cr_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_cr_detect = TRUE;
+			le_u_crf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == A_CR) && (c2 == 0) && 
+		    (c3 == 0) && (c4 == 0) &&
+			(in_big_endian || (icw == 3))) {
+		    set_in_big_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_cr_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_cr_detect = TRUE;
+			le_u_crf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == 0) && (c2 == A_CR) && 
+			is_ucs_utf16(i_codeset[in_codeset].encode) &&
+			(!in_big_endian || (icc == 1))) {
+		    set_in_ltl_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_cr_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_cr_detect = TRUE;
+			le_u_crf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == A_CR) && (c2 == 0) && 
+			is_ucs_utf16(i_codeset[in_codeset].encode) &&
+			(in_big_endian || (icc == 1))) {
+		    set_in_big_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_cr_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_cr_detect = TRUE;
+			le_u_crf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == 0) && (c2 == 0) && 
+		    (c3 == A_LF) && (c4 == 0) &&
+			(!in_big_endian || (icw == 3))) {
+		    set_in_ltl_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_lf_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_lf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == A_LF) && (c2 == 0) && 
+		    (c3 == 0) && (c4 == 0) &&
+			(in_big_endian || (icw == 3))) {
+		    set_in_big_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_lf_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_lf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == 0) && (c2 == A_LF) && 
+			is_ucs_utf16(i_codeset[in_codeset].encode) &&
+			(!in_big_endian || (icc == 1))) {
+		    set_in_ltl_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_lf_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_lf_detect = TRUE;
+		    };
+	    	} else if ((rc1 == A_LF) && (c2 == 0) && 
+			is_ucs_utf16(i_codeset[in_codeset].encode) &&
+			(in_big_endian || (icc == 1))) {
+		    set_in_big_endian;
+		    if (le_u_cr_detect || le_u_lf_detect) {
+		    	le_u_lf_detect = TRUE;
+			set_le_parse(le_u_cr_detect,le_u_lf_detect,
+					le_u_crf_detect);
+			/* two le char detected. confirmed. */
+			add_rescase();
+			enque(rc1);
+			if (is_code_undefine) continue;
+			else break;
+		    } else {
+		    	le_u_lf_detect = TRUE;
+		    };
+		} else ;
 #ifdef SKFDEBUG
 		if (is_vv_debug) {
 		  if (detect_cr) fprintf(stderr," CR_n trap ");
 		  if (detect_lf) fprintf(stderr," LF_n trap ");
 		};
 #endif
-	    };
-	  } else;
+	    } else;	/* already knows le type.  */
+	  } else;	/* not UTF-16 */
 #endif
 /* ---------------------------------------------------- */
 /* text types						*/
@@ -1007,7 +1101,14 @@ int preConvert (f)
 /* !ucs2						*/
 /* ---------------------------------------------------- */
 	  if (is_code_undefine
-		|| (in_codeset != codeset_utf16le)) {
+		|| ((in_codeset != codeset_utf16le) &&
+		    (in_codeset != codeset_utf16lenb) &&
+		    (in_codeset != codeset_utf16be) &&
+		    (in_codeset != codeset_utf16benb) &&
+		    (in_codeset != codeset_utf32le) &&
+		    (in_codeset != codeset_utf32be) &&
+		    (in_codeset != codeset_utf32lenb) &&
+		    (in_codeset != codeset_utf32benb))) {
 	    if (rc1 == A_CR) {
 		le_n_cr_detect = TRUE;
 		if (!le_n_lf_detect) le_n_crf_detect = TRUE;
@@ -1079,14 +1180,14 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 		ascii_code_only = FALSE; in_utf7_encode = 0;
 		keisfault++; utf8fault++; hzfault++; latinfault++;
 		utf7fault++; ucs2fault++; sjisfault++; eucfault++;
-		vscfault++; vsqfault++;
+		vscfault++; vsqfault++; b5fault++;
 		e7upper = 0; gblikely++; krlikely++;
 	    } else if (c1 == A_SO) {
 		jisccnt = 0;
 		ascii_code_only = FALSE; in_utf7_encode = 0;
 		keisfault++; utf8fault++; hzfault++; latinfault++;
 		utf7fault++; ucs2fault++; sjisfault++; eucfault++;
-		vscfault++; vsqfault++;
+		vscfault++; vsqfault++; b5fault++;
 		e7upper = 1; gblikely++; krlikely++;
 		eucccnt = 0;
 	    } else if (c1 == A_SS2) { /* euc-jp			   */
@@ -1104,7 +1205,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 		latinfault++; keisfault++; utf7fault++; utf8fault++;
 		if (zeroappear >= det_limit) {/* 4th zero appears  */
 		    set_rescase(152);
-		    in_codeset = codeset_utf16le;
+		    in_codeset = codeset_utf16lenb;
 		    if (normalcode <= 1) { /* explicitly UCS-4	   */
 			set_in_big_endian; 	/* seems big	   */
 		    } else if (normalcode < 3) {
@@ -1143,7 +1244,9 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 		/* Note: 0xa0 is regarded as kana		   */
 		jisprech = jiscurch; 
 		jiscurch = c1 - 0xa0 + 0xff60;
-		if (!input_x201_kana && (jisfault == 0)) {
+		if (!input_x201_kana) {
+		    jisfault++; sjisfault++;
+		} else if (jisfault == 0) {
 		    kanachar++;
 		    if (c1 == 0xa1) {
 			kanakuten++;
@@ -1277,14 +1380,14 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	      if (Qempty && (c1 == 0xfe) && (c2 == 0xff) 
 			  && is_lang_unicode) {
 		  set_rescase(130);
-		  in_codeset = codeset_utf16le;
+		  in_codeset = codeset_utf16lenb;
 		  has_bom = TRUE;
 		  if (!(endian_protect)) set_in_ltl_endian ;
 		  enque(rc1); continue;
 	      } else if (Qempty && (c1 == 0xff) && (c2 == 0xfe)
 			  && is_lang_unicode) {
 		  set_rescase(131);
-		  in_codeset = codeset_utf16be;
+		  in_codeset = codeset_utf16benb;
 		  has_bom = TRUE;
 		  if (!(endian_protect)) set_in_big_endian ;
 		  enque(rc1); continue;
@@ -1332,17 +1435,15 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	/* check intermediate seqs. */
 /* --- UTF-8 ------------------ */
 	if (is_lang_unicode && (utf8fault == 0)) {
-	    if (Qle2 && (c3 == 0xef) && (c2 == 0xbb) && (rc1 == 0xbf)) {
+	    if ((c2 == 0xbb) && (rc1 == 0xef)) { /* utf8	   */
 		utf8likely += 3;	/* UTF8 BOM		   */
 		utf8prech = sEOF; utf8curch = sEOF;
 		has_bom = TRUE;
-	    } else if (Qempty && (c2 == A_ESC) && (rc1 > 0x20) && (rc1 < 0x30)) {
+	    } else if ((c2 == A_ESC) && (rc1 > 0x20) && (rc1 < 0x30)) {
 		utf8fault++;
 	    } else if ((utf8curch = is_valid_utf8_seq(rc1,c2,c3,c4)) >= 0) {
-		if (utf8curch == 0) {
+		if ((utf8curch == 0) || (rc1 < A_DEL)) {
 		    utf8ccnt = 1;
-		} else if (rc1 < A_DEL) {
-		    utf8ccnt = 0;
 		} else {
 		    utf8ccnt = 0;	/* get one utf8 char	   */
 		    utf8likely++;
@@ -1418,7 +1519,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	    } else;
 	    if ((c1 == 0xa4) || ((c1 >= 0xb0) && (c1 <= 0xcf))) {
 		euclikely++;	/* hiragana and 1st-level kanji	   */
-	    };
+	    } else if ((c1 == 0xa2) && (c2 == 0xae)) euclikely++;
   /* --- euc-cn --- */
 	    if ((c1 >= 0xac) && (c1 <= 0xaf)) gbfault++;
 	    if (c1 >= 0xf8) gbfault++;
@@ -1585,30 +1686,35 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 		sjiscurch = c1 - 0xa0 + 0xff60;
 		if (!input_x201_kana && (sjisfault == 0)) {
 		    kanachar++;
+		    if (c2 == 0xa1) kanakuten++;
+		    if (c2 == 0xa4) kanatouten++;
 		    if (c1 == 0xa1) {
 			kanakuten++;
-			if ((c2 == 0xa3) || (c2 == 0x29)) {
+			if ((c2 == 0xa3) || (c2 == 0x29) || (c2 == 0xa1)) {
 			    sjislikely++; }
-			else if (c2 <= 0xb1) { sjisfault++;}
+			else if (c2 <= 0xae) { sjisfault++;}
 			else ;
-		    };
-		    if (c2 == 0xa1) kanakuten++;
-		    if (c1 == 0xa2) {
+		    } else if (c1 == 0xa2) {
 			if ((c2 == 0xa2) ||((c2 > 0xa4) && (c2 < 0xb0))) 
 			    sjisfault++;
-		    };
-		    if (c1 == 0xa3) {
+		    } else if (c1 == 0xa3) {
 			if ((c2 == 0xa3) ||((c2 > 0xa1) && (c2 < 0xb0))) 
 			    sjisfault++;
-		    };
-		    if (c1 == 0xa4) {
+		    } else if (c1 == 0xa4) {
 			kanatouten++;
 			if ((c2 == 0xa3) || (c2 == 0x29)) {
 			    sjislikely++;}
 			else if (c2 <= 0xa6) { sjisfault++;}
 			else ;
+		    } else if ((c1 == 0xae) && (c2 == 0xa2)) {
+		    	sjisunlikely++;
 		    };
-		    if (c2 == 0xa4) kanatouten++;
+		    if (c1 == 0xde) {
+		    	if ((c1 <= 0xb5) || ((c1 >= 0xc5) && (c1 <= 0xc9))
+				|| (c1 >= 0xcf)) sjisfault++;
+		    } else if (c1 == 0xdf) {
+		    	if ((c1 <= 0xc9) || (c1 >= 0xcf)) sjisfault++;
+		    } else;
 		    if ((kanachar > 30) && (kanakuten == 0)
 			&& (kanatouten == 0)) {
 			kanaunlikely++;
@@ -1653,6 +1759,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 #ifdef KEIS_DETECT
 /* --- KEIS --- */
 	  if ((keisfault == 0) && (!keis_detect_untest)) {
+	    if ((rc1 == 0x0e) || (rc1 == 0x0f)) keisfault++;
 	    if (rc1 > MS_END) {  /* 0xeb-0xfd			   */
 		if ((rc1 > 0xf9) || (rc1 < 0xf0)
 		    || (ebcdik40[c2] == 0) || (ebcdik40[c2] >= 0x100)) {
@@ -1694,7 +1801,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	  };
 #else
 	    if (keisfault == 0) {
-		keislikely = 0;
+		keislikely = 0;	
 		keisfault++;
 		keisunlikely++;
 		debug_fault("keis",802);
@@ -1914,7 +2021,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 /* plausible test */
 	if (zeroappear > 0) {
 	    set_rescase(400);		/* detect as UCS2	   */
-	    in_codeset = codeset_utf16le;
+	    in_codeset = codeset_utf16lenb;
 	} else if ((!eucfault) && (eucunlikely == 0) 
 		&& (euclikely > det_limit)) {
 	    set_rescase(401); 		/* detect as EUC	   */
@@ -1997,16 +2104,16 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	    set_rescase(431);
 	    in_codeset = codeset_viscii;
 /* 3rd try: compare likelyhood */
-	} else if ((sjislikely > euclikely)
-		|| ((sjislikely > 0) && eucfault)) {
+	} else if (((sjislikely > euclikely)
+		|| ((sjislikely > 0) && eucfault)) & !sjisfault) {
 	    set_rescase(440);
 	    in_codeset = codeset_sjis;
-	} else if ((utf8likely > euclikely) 
-		|| ((utf8likely > 0) && eucfault && sjisfault)) {
+	} else if (((utf8likely > euclikely) 
+		|| ((utf8likely > 0) && eucfault && sjisfault)) & !utf8fault) {
 	    set_rescase(441);
 	    in_codeset = codeset_utf8;
-	} else if ((euclikely > sjislikely) 
-		|| ((euclikely > 0) && utf8fault)) {
+	} else if (((euclikely > sjislikely) 
+		|| ((euclikely > 0) && utf8fault)) & !eucfault) {
 	    set_rescase(442);
 	    in_codeset = codeset_eucjp;
 /* No clue ... */
@@ -2121,7 +2228,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 		printf("%s: ",in_file_name);
 	};
 #endif
-	dump_name_of_code(was_ascii,1, has_bom);
+	dump_name_of_code(was_ascii,0, has_bom);
 #ifndef SWIG_EXT
 	if (input_hard_inquiry) {
 	    dump_name_of_lineend(le_detect,0);
@@ -2132,7 +2239,6 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	return(sEOF);
     };
 
-#if	defined(NKF_COMPATMODE)
 /* --- nkf-feature: CRLF normalize ---- */
     if (is_nkf_compat && is_lineend_normalize && is_o_encode) {
     	if (detect_cr) {
@@ -2142,7 +2248,8 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	    set_lineend_lf;
 	} else;
     } else;
-#endif
+/* --- used lineend definition --- */
+    le_defs = le_detect;
 
     /* Now we know the code, and ready for input process. Then set */
     /* everything for input process(tables etc.) at this point.	   */
@@ -2192,7 +2299,7 @@ int	has_bom;
     } else codename = dcode_name;
 
     if (out == 1) {
-    	fprintf(stderr,"%s\n",codename);
+    	fprintf(stderr,"%s",codename);
     } else {
     	SKFSTROUT(codename);
     };	
