@@ -1,5 +1,5 @@
 /* ------------------------------------------------------------------
-** Copyright (c) 1993-2012 Seiji Kaneko. All rights reserved.
+** Copyright (c) 1993-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -25,7 +25,7 @@
     skf.h:		common definitions
 	v1.30		first written for skf v1.30
 	v1.90 		written for skf v1.9
-    $Id: skf.h,v 1.176 2013/07/22 13:42:44 seiji Exp seiji $
+    $Id: skf.h,v 1.179 2015/04/26 15:08:38 seiji Exp seiji $
 */
 
 #ifndef		INCLUDE_SKFDEFS
@@ -68,7 +68,6 @@
 #define         A_CR	0x0d
 #define         A_SO	0x0e
 #define         A_SI	0x0f
-#define         A_NEL	0x15
 #define         A_CAN	0x18
 #define         A_SUB	0x1a
 #define         A_ESC	0x1b
@@ -77,9 +76,9 @@
 #define		A_BANG	0x21
 #define         A_COMMA	0x2c
 #define         A_DOT	0x2e
+#define         A_0	0x30
 #define         A_BRA	0x3c
 #define         A_KET	0x3e
-#define         A_QUE	0x3f
 #define         A_AT	0x40
 #define         A_CAPA	0x41
 #define         A_CAPZ	0x5a
@@ -111,7 +110,6 @@
 #define		ASCII_END 0x80
 #define		A_KDEL	0xff
 #define		MS_GAIJI 0xf0
-#define		UCODE_REPLACE 0xfffd
 
 /* X-0208 assignment						   */
 #define		A_MAX	0x28	/* maximum alpha code		   */
@@ -227,7 +225,6 @@
 #define		LBMSK	0x0ffU
 #define		HBMSK	0xff00U
 
-#define FIXSTRLEN_LIMIT	64
 /* --------------------------------------------------------------  */
 /* internal types						   */
 /* --------------------------------------------------------------  */
@@ -422,7 +419,6 @@ extern short	debug_opt;
 #define SKF_GBKR	0x00a0UL    /* GBKR-derivative style codes */
 #define SKF_NNISO	0x00c0UL    /* Various misc code sets	   */
 #define SKF_KEIS 	0x00e0UL    /* output is kind of KEIS	   */
-#define SKF_CDSP 	0x00f0UL    /* output is not ASCII/EBCDIC  */
 
 /* --------------------------------------------------------------- */
 /* misc selectors 0x0000000*					   */
@@ -501,12 +497,14 @@ extern short	debug_opt;
 #define SKF_UCS_UTF7	0x0006UL    /* BASE utf-7		   */
 #define SKF_UCS_PUNY	0x0008UL    /* BASE IDN Punycode	   */
 #define SKF_UCS_URI	0x0009UL    /* BASE IDN URI notation	   */
+#define SKF_UCS_BRGT	0x000eUL    /* output is B/Right V	   */	
 #define SKF_UCS_TRANS	0x000fUL    /* transparent mode		   */
 
 #define is_ucs_utf16(x) ((x & SKF_COD_UC_MSK)==(SKF_UCS|SKF_UCS_UTF16))
 #define is_ucs_utf32(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_UTF32))
 #define is_ucs_utf8(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_UTF8))
 #define is_ucs_utf7(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_UTF7))
+#define is_ucs_brgt(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_BRGT))
 #define is_ucs_puny(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_PUNY))
 #define is_ucs_uri(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_URI))
 #define is_ucs_transp(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_TRANS))
@@ -639,15 +637,6 @@ extern short	debug_opt;
 #define is_keis_melc(x) ((x & SKF_COD_O_MSK) == (SKF_KEIS | SKF_MELC))
 #define is_keis_keis(x) ((x & SKF_COD_O_MSK) == SKF_KEIS)
 
-/* --------------------------------------------------------------- */
-/* 10th set: SKF_CDSP						   */
-/* --------------------------------------------------------------- */
-#define out_cdsp  ((conv_cap & SKF_COD_C_MSK) == SKF_CDSP)
-
-#define SKF_CDSP_BRGT	0x00000001UL    /* output is B/Right V	  */	
-
-#define is_brgt(x)	((x & SKF_COD_O_MSK) == (SKF_CDSP | SKF_CDSP_BRGT))
-
 #define is_cod_iso(x) ((((x & SKF_COD_CAT_MSK) == 0) || is_msfam(x) \
 			|| is_hz(x) || is_zW(x)) \
 			&& !is_euc_niso(x))
@@ -733,7 +722,7 @@ extern int	skip_x221_4_null();
 #define	USE_JIS8 0x00001000UL	/* iso-2022 8bit enable		   */
 
 /* --------------------------------------------------------------- */
-/* should not be needed.					   */
+/* should be not needed.					   */
 /* --------------------------------------------------------------- */
 #define COD_S_90  0x00000400UL	/* code is not JIS X0208(1990)	   */
 #define sup_jis90 (conv_alt_cap & COD_S_90)
@@ -777,15 +766,22 @@ extern int	skip_x221_4_null();
 #define set_fold_flat conv_alt_cap |= FOLD_FLAT
 #define reset_fold_flat conv_alt_cap &= 0xfffffffbUL
 
-#define FOLD_STRONG 0x00000002UL /* strong kinsoku		   */
+#define FOLD_STRONG 0x00000001UL /* strong kinsoku		   */
 #define fold_strong (conv_alt_cap & FOLD_STRONG)
 #define set_fold_strong conv_alt_cap |= FOLD_STRONG
 #define reset_fold_strong conv_alt_cap &= 0xfffffffdUL
 
+#ifdef NOSPACECONV
+#define NO_SUP_SPACE_CONV 0x00000001UL /* ideographic space through mode */
+#define sup_space_conv !(conv_alt_cap & NO_SUP_SPACE_CONV)
+#define res_sup_space_conv conv_alt_cap |= NO_SUP_SPACE_CONV
+#define set_sup_space_conv conv_alt_cap &= 0xfffffffeUL
+#else
 #define SUP_SPACE_CONV 0x00000001UL /* ideographic space through mode */
 #define sup_space_conv (conv_alt_cap & SUP_SPACE_CONV)
 #define set_sup_space_conv conv_alt_cap |= SUP_SPACE_CONV
 #define res_sup_space_conv conv_alt_cap &= 0xfffffffeUL
+#endif
 
 /* --------------------------------------------------------------  */
 /* preconvert.c control: 					   */
@@ -1011,11 +1007,14 @@ extern int 	o_encode;
 #define	O_ENCODE_URI	0x0200U
 #define	O_ENCODE_DCM	0x0400U	/* not used for output		   */
 #define	O_ENCODE_Q	0x0800U /* hex but uses "=" 		   */
-#define	O_ENCODE_PUNY	0x1000U /* punycode			   */
+#define	O_ENCODE_PUNY	0x1000U /* hex but uses "=" 		   */
+
+#define O_ENCODE_MIMES  (O_ENCODE_MIME | O_ENCODE_MIMEQ | O_ENCODE_RFC2231)
+#define O_ENCODE_MIMEBQ  (O_ENCODE_MIME | O_ENCODE_MIMEQ)
 
 #define O_ENCODE_MIME_MASK 0x001cU
 #define O_ENCODE_HEX_MASK 0x0f03U
-#define O_ENCODE_IUCS2	   0x007eU
+#define O_ENCODE_IUCS2	   0x0082U
 
 /*
  * rfc2231, dcm is NOT implemented.
@@ -1034,7 +1033,8 @@ extern int 	o_encode;
 #define is_o_encode_cap(x) \
 	((x & O_ENCODE_HEX_MASK) == O_ENCODE_HEX)
 #define is_o_encode		(o_encode != 0)
-#define is_o_encode_mime(x)	((x & O_ENCODE_MIME) || (x & O_ENCODE_MIMEQ))
+#define is_o_encode_mimebq(x)	(x & O_ENCODE_MIMEBQ)
+#define is_o_encode_mimes(x)	(x & O_ENCODE_MIMES)
 
 #define set_o_mimeb_encode o_encode = O_ENCODE_MIME
 #define set_o_mimeq_encode o_encode = O_ENCODE_MIMEQ
@@ -1045,6 +1045,7 @@ extern int 	o_encode;
 #define set_o_oct_encode o_encode = O_ENCODE_OCT
 #define set_o_base64_encode o_encode = O_ENCODE_BASE64
 #define set_o_2231_encode o_encode = O_ENCODE_RFC2231
+#define reset_o_encode o_encode = 0
 
 /* --------------------------------------------------------------  */
 /* Encoding control: input side					   */
@@ -1060,6 +1061,7 @@ extern	long		encode_cap;
 
 /* --------------------------------------------------------------  */
 #define set_hex_encoded	encode_cap = O_ENCODE_HEX
+#define reset_encoded encode_cap = 0;
 #define decode_set(x) encode_cap = x
 
 #define is_hex_encoded	(encode_cap & O_ENCODE_HEX)
@@ -1102,6 +1104,7 @@ extern	long		encode_cap;
 /* fold and lineend controls					   */
 /* --------------------------------------------------------------  */
 extern int	le_detect;	/* lineend detect marker	   */
+extern int	le_defs;	/* normalized lineend		   */
 
 #define set_detect_cr	le_detect |= 0x02
 #define set_detect_lf	le_detect |= 0x04
@@ -1109,12 +1112,22 @@ extern int	le_detect;	/* lineend detect marker	   */
 #define set_first_detect_cr le_detect |= 0x10
 #define set_mix_detect	le_detect = 0x20
 #define set_dummy_detect le_detect |= 0x100
+#define set_add_lastle le_detect |= 0x1000
 
 #define detect_cr	(le_detect & 0x02)
 #define detect_lf	(le_detect & 0x04)
 #define first_detect_cr (le_detect & 0x10)
 #define mix_le_detect	(le_detect & 0x20)
 #define is_dummy_ledet  (le_detect & 0x100)
+#define is_add_lastle	(le_detect & 0x1000)
+
+#define use_detect_cr	(le_defs & 0x02)
+#define use_detect_lf	(le_defs & 0x04)
+#define use_first_detect_cr	(le_defs & 0x10)
+#define set_use_detect_cr	le_defs |= 0x02
+#define set_use_detect_lf	le_defs |= 0x04
+#define set_use_detect_crlf	le_defs |= 0x06
+#define set_use_first_detect_cr le_defs |= 0x10
 
 /* --------------------------------------------------------------  */
 /* option guarding controls: force respect command line option.	   */
@@ -1207,7 +1220,7 @@ extern unsigned long	ucod_flavor;
 #define enable_dbl_latin (ucod_flavor & ENBL_DBL_LATIN)
 #define set_enable_dbl_latin ucod_flavor |= ENBL_DBL_LATIN
 
-#define OLDCELL_TO_EMOT 0x00001000UL
+#define OLDCELL_TO_EMOT 0x00020000UL
 #define enable_cellconvert (ucod_flavor & OLDCELL_TO_EMOT)
 #define set_enable_cellconvert ucod_flavor |= OLDCELL_TO_EMOT
 
@@ -1219,6 +1232,11 @@ extern unsigned long	ucod_flavor;
 /* mainly enables emoji-area conversion				   */
 #define set_nounicode6  ucod_flavor |= ENBL_NOUNICODE6
 #define is_nounicode6  (ucod_flavor & ENBL_NOUNICODE6)
+
+#define NYUKAN_CONVERT 0x00100000UL
+#define nyukan_convert (ucod_flavor & NYUKAN_CONVERT)
+#define set_nyukan_convert ucod_flavor |= NYUKAN_CONVERT
+
 /* --------------------------------------------------------------  */
 /* Note: 0x***0**** - disable language tag completely		   */
 /*       0x***1**** - output language tag if strong lang specified */
@@ -1249,8 +1267,8 @@ extern int	unbuf_f;	/* unbuffered mode		   */
 extern short    debug_opt;	/* debug level			   */
 extern skf_ucode ucode_undef;	/* unic*de undef char		   */
 
-/* kanji-kana trasition character definition			   */
-extern char     k_in,k_out;
+/* kanji-kana transition character definition			   */
+extern char	k_in,k_out;
 
 /* --------------------------------------------------------------  */
 /* codeset flavors controls					   */
@@ -1294,10 +1312,10 @@ extern unsigned long codeset_flavor;
 #define add_renew (codeset_flavor & ADD_RENEW)
 #define set_add_renew codeset_flavor |= ADD_RENEW
 
-#define USE_COMPAT  0x00000020UL  /* use unic*de compatible plane  */
-#define use_compat (codeset_flavor & USE_COMPAT)
-#define set_use_compat codeset_flavor |= USE_COMPAT
-#define reset_use_compat codeset_flavor &= 0xffffffdfUL
+#define UNUSE_COMPAT  0x00000020UL  /* unuse unic*de compatible plane  */
+#define use_compat !(codeset_flavor & UNUSE_COMPAT)
+#define reset_use_compat codeset_flavor |= UNUSE_COMPAT
+#define set_use_compat codeset_flavor &= 0xffffffdfUL
 
 #define USE_CDE_COMPAT 0x00000010UL
 #define use_cde_compat (codeset_flavor & USE_CDE_COMPAT)
@@ -1385,11 +1403,9 @@ extern unsigned long	nkf_compat;
 #define set_lineend_normalize nkf_compat = nkf_compat | LINEEND_NORMALIZE
 #define set_mime_nkfmode nkf_compat |= MIME_UNTRUNC_SPACE
 
-#ifdef NKF_COMPATMODE
 #define set_nkf_compat	nkf_compat |= (NKF_FULLCOMPAT | LINEEND_NORMALIZE)
 #define reset_nkf_compat nkf_compat &= 0x00ffffffUL
 #define is_nkf_compat	(nkf_compat & NKF_FULLCOMPAT)
-#endif	/* NKF_COMPATMODE */
 
 /* fullwidth space to space control */
 #define set_spconv_1	nkf_compat |= SPACE_CONV_1
@@ -1404,7 +1420,6 @@ extern unsigned long	nkf_compat;
 #define is_kanaconv_x0201 (nkf_compat & KANA_CONV_X0201)
 #define is_ascii_conv (nkf_compat & ASCII_CONV_X0208)
 
-#ifdef NKF_COMPATMODE
 #define set_cp932_ext	nkf_compat |= NKF_CP932_EXT
 #define set_nocp932_ext	nkf_compat |= NKF_NOCP932_EXT
 #define set_nocp932	nkf_compat |= NKF_NOCP932
@@ -1421,7 +1436,6 @@ extern unsigned long	nkf_compat;
 #define set_nkf_jbroken nkf_compat |= INPUT_JIS_BROKEN
 #define set_nkf_jfbroken nkf_compat |= INPUT_JIS_FBROKEN
 #define set_nkf_jffbroken nkf_compat |= INPUT_JIS_LBROKEN
-#endif	/* NKF_COMPATMODE */
 
 #define set_notrunc_le	nkf_compat |= FOLD_NOTRUNC_LE
 #define notrunc_le	(nkf_compat & FOLD_NOTRUNC_LE)
@@ -1445,8 +1459,8 @@ extern unsigned long	nkf_compat;
 #define set_nkf_c_katakana nkf_compat |= NKF_KATAKANA
 
 #define is_nkf_no_hk	(nkf_compat & NKF_FORCE_NOHK)
-#define set_nkf_no_hk	nkf_compat = (nkf_compat & 0xfdffffffUL) | NKF_FORCE_NOHK
-#define reset_nkf_no_hk	nkf_compat = (nkf_compat & 0xfdffffffUL) 
+#define set_nkf_no_hk	nkf_compat = (nkf_compat & 0xffefffffUL) | NKF_FORCE_NOHK
+#define reset_nkf_no_hk	nkf_compat = (nkf_compat & 0xffefffffUL) 
 #define is_o_prefer_ascii	(nkf_compat & O_PREFER_ASC)
 #define set_o_prefer_ascii nkf_compat |= O_PREFER_ASC
 /* -------------------------------------------------------------- */
@@ -1591,7 +1605,8 @@ extern int	mime_fold_llimit;
 #define L_IT		0x4954		/* Itarian		  */
 #define L_IU		0x4955		/* Inuktitut		  */
 #define	L_JA		0x4a41		/* Japanese		  */
-#define	L_JP		0x4a41		/* Japanese		  */
+#define	L_JI		0x4a49		/* Jidish		  */
+#define	L_JP		0x4a50		/* Japanese		  */
 #define	L_JV		0x4a56		/* Javanese		  */
 #define L_KA		0x4b41		/* Georgian		  */
 #define L_KG		0x4b47		/* Kongo		  */
@@ -1710,10 +1725,25 @@ extern int	mime_fold_llimit;
 #define N_CN		0x434e		/* Continental China	  */
 #define N_HK		0x484b		/* HongKong		  */
 #define N_JA		0x4a41		/* Japan		  */
+#define N_JP		0x4a50		/* Japan		  */
 #define N_KR		0x4b52		/* Koria		  */
 #define N_PT		0x5054		/* Portugal		  */
 #define N_RU		0x5255		/* Russia		  */
 #define N_TW		0x5457		/* Taiwan		  */
+#define N_US		0x5553		/* United States	  */
+
+#define N_JAJP		0x4a414a50	/* ja_JP		  */
+#define N_ZHTW		0x5a485458	/* zh_TW		  */
+#define N_ZHCN		0x5a48434e	/* zh_CN		  */
+#define N_ENUS		0x454e5553	/* en_US		  */
+#define N_ENGB		0x454e4742	/* en_GB		  */
+#define N_KOKR		0x4b4f4b52	/* ko_KR		  */
+#define N_PTPT		0x50545054	/* pt_PT		  */
+#define N_PTBR		0x50544252	/* pt_BR		  */
+#define N_THTH		0x55485548	/* th_TH		  */
+#define N_VIVN		0x5649564f	/* vi_VN		  */
+#define N_RURU		0x52555255	/* ru_RU		  */
+#define N_RUUA		0x52555541	/* ru_UA		  */
 /* -------------------------------------------------------------- */
 
 #define LANGCD_MSK	0xdfdfUL
@@ -1827,7 +1857,7 @@ unsigned long skf_get_valid_langcode P_((unsigned long,unsigned long));
 #define	codeset_natsf	80  /* NATS for Finland/Sweden		  */
 #define	codeset_natsd	81  /* NATS for Denmark/Norway		  */
 #define codeset_jp2latin 82 /* Reserved				  */
-#define codeset_cp1250   83 /* GL: ascii GR: cp1251		  */
+#define codeset_cp1250   83 /* GL: ascii GR: cp1250		  */
 #define codeset_cp1251   84 /* GL: ascii GR: cp1251		  */
 #define	codeset_bs4730	 85  /* GB BS 4730			  */
 #define	codeset_nfz62010 86  /* French NF Z 62010		  */
@@ -1866,9 +1896,16 @@ unsigned long skf_get_valid_langcode P_((unsigned long,unsigned long));
 #define codeset_sjisau	 119 /* Shift JIS with AU glyph		  */
 #define codeset_sjissb	 120 /* Shift JIS with S*ftBank glyph	  */
 #define codeset_sjisontt 121 /* Shift JIS with old-ntt glyph	  */
-#define	codeset_binary	 122  /* no codeset is given		  */
-#define	codeset_locale	 123  /* no codeset is given		  */
-#define codeset_end	 124 /* psuedo code: end-point		  */
+#define	codeset_nyukan_utf8 122  /* nyukan gaiji utf8		  */
+#define	codeset_nyukan_utf16 123  /* nyukan gaiji utf8		  */
+#define	codeset_binary	 124  /* no codeset is given		  */
+#define	codeset_locale	 125  /* no codeset is given		  */
+#ifdef ARIB_SUPPORT
+#define	codeset_aribb24	 126  /* ARIB-B24-SI			  */
+#define codeset_end	 127 /* psuedo code: end-point		  */
+#else
+#define codeset_end	 126 /* psuedo code: end-point		  */
+#endif
 
 /* -------------------------------------------------------------- */
 struct long_option {
@@ -1882,6 +1919,12 @@ struct long_option {
 	(in_codeset == codeset_transp) || \
 	(in_codeset == codeset_utf7) || \
 	(in_codeset == codeset_puny))
+
+#define is_in_ucs_ufamnomime ((in_codeset == codeset_utf16le) || \
+	(in_codeset == codeset_utf16be) || \
+	(in_codeset == codeset_transp) || \
+	(in_codeset == codeset_utf7) || \
+	(in_codeset == codeset_puny))
 /* -------------------------------------------------------------- */
 /* iscii character properties					  */
 /* -------------------------------------------------------------- */
@@ -1912,7 +1955,6 @@ extern const struct long_option codeset_option_code[];
 /* -------------------------------------------------------------- */
 extern int	cname_comp	P_ ((char *, char *));
 extern int 	skf_search_cname P_((char *));
-extern int 	skf_search_cname_w_alias P_((char *));
 extern int 	skf_search_chname P_((char *));
 extern int	skf_option_parser \
 			P_((char *,const struct long_option *));
@@ -1934,7 +1976,6 @@ extern  void	print_annouce P_((int));    /*			  */
 extern	void	o_c_encode P_((int));
 extern	void	o_p_encode P_((skf_ucode));
 extern  int	mime_clip_test P_((int,int));
-extern	char	*alt_desc;
 
 /* -------------------------------------------------------------- */
 /* post oconv side						  */
@@ -2075,20 +2116,19 @@ extern void	utf8_finish_procedure();
 extern void	ucod_finish_procedure();
 extern void	BG_finish_procedure();
 extern void	BRGT_finish_procedure();
-extern void	KEIS_finish_procedure();
-extern void	puny_finish_procedure();
-extern void	GBKR_finish_procedure();
 
 /* -------------------------------------------------------------- */
 /* windows filename limitation hooker				  */
 /* -------------------------------------------------------------- */
 extern const char	charname_conv[];
+extern const char	charname_priv_conv[];
 
 /* -------------------------------------------------------------- */
 /* error code assist: in error.c				  */
 /* -------------------------------------------------------------- */
 extern void	display_help();
 extern void	display_version P_((int));
+extern void	display_nkf_help();
 
 extern void	skferr P_((int, long, long));
 extern void	skf_openerr P_((char *,int));
@@ -2113,7 +2153,11 @@ extern void	ValidValueDisplay P_((int, char *));
 extern void	ValidLangDisplay P_((char *));
 extern void	dump_name_of_lineend P_((int, int));
 
-extern	char	*in_file_name;	/* saved input file name	  */
+extern char	*in_file_name;	/* saved input file name	  */
+
+#ifndef SWIG_EXT
+extern int	get_output_locale();
+#endif
 
 /* in/out_undefined() */
 #define SKF_IUNDEF	1	/* unsupported input codepoint	   */
