@@ -1,5 +1,5 @@
 /* *******************************************************************
-** Copyright (c) 1999-2009 Seiji Kaneko.  All rights reserved.
+** Copyright (c) 1999-2015 Seiji Kaneko.  All rights reserved.
 ** Everyone is permitted to use this program in source and binary
 ** form, with or without modification if and only if the following
 ** conditions are met:
@@ -23,7 +23,7 @@
 **********************************************************************
     ucodoconv.c: output converters for Unicode(TM) variants
 	v1.90	Newly written from unic*de.c parsing
-    $Id: ucodoconv.c,v 1.77 2013/07/22 13:42:44 seiji Exp seiji $
+    $Id: ucodoconv.c,v 1.80 2015/04/26 15:08:38 seiji Exp seiji $
 **/
 
 #include <stdio.h>
@@ -48,10 +48,13 @@ static int	utf7_res = 0;	/* utf-7 amari			   */
 static void SKFUTF7ENCODE P_((skf_ucode));
 static void encode_pslenset P_((int *,int *,skf_ucode));
 
+static char ucod_cbuf[32];	/* uri conversion buffer	   */
+
 #ifdef ACE_SUPPORT
-int  is_prohibit P_((skf_ucode));
+static int  is_prohibit P_((skf_ucode));
 #endif
 
+extern char *utf8_urioutstr	P_((skf_ucode));
 /* --- macros ---------------------------------------------------- */
 #define utf8put4(x) SKFputc((int)(ZF4VAL + ((x & ZL4MSK) >> 18))); \
     SKFputc((int)(ZF2VAL + ((x & ZL43MSK) >> 12)));\
@@ -145,26 +148,26 @@ void SKFUNISTROUT(st)
 const char *st;
 {
     int len;
-    for (len=0; (len<30) && (*st != '\0') ; len++,st++) {
+    for (len=0; (len<24) && (*st != '\0') ; len++,st++) {
 	SKFUNI1OUT((skf_ucode)(*st));
     };
 }
 
 /* --- ascii output ---------------------------------------------- */
-void UNI_ascii_oconv(c2)
-    skf_ucode	c2;
+void UNI_ascii_oconv(ch)
+    skf_ucode	ch;
 {
     int		c1,c3;
 
-    c1 = c2 & 0xff;	/* guarding				   */
+    c1 = ch & 0xff;	/* guarding				   */
     c3 = uni_o_ascii[c1];
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," uni_ascii:%02x",c2);
+    if (is_vv_debug) fprintf(stderr," uni_ascii:%02x",ch);
 #endif
     if (o_encode) {
-    	out_UNI_encode(c2,c3);
-	if (is_ucs_utf16(conv_cap) && is_lineend(c2)) {
-	    SKFrputc(c2 & 0xff);
+    	out_UNI_encode(ch,c3);
+	if (is_ucs_utf16(conv_cap) && is_lineend(ch)) {
+	    SKFrputc(ch & 0xff);
 	    return;
 	} else;
     } else;
@@ -185,17 +188,17 @@ void UNI_ascii_oconv(c2)
 
 /* --- latin output ---------------------------------------------- */
 /*  handles 0xa0-0x2fff						   */
-void UNI_latin_oconv (c3)
-    skf_ucode    c3;
+void UNI_latin_oconv (ch)
+    skf_ucode    ch;
 {
     int c1,c2;
 
-    c1 = (c3 & 0xff);
-    c2 = (c3 >> 8) & 0xff;
+    c1 = (ch & 0xff);
+    c2 = (ch >> 8) & 0xff;
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," uni_latin:%04x",c3);
+    if (is_vv_debug) fprintf(stderr," uni_latin:%04x",ch);
 #endif
-    if (o_encode) out_UNI_encode(c3,c3);
+    if (o_encode) out_UNI_encode(ch,ch);
 
     if (is_ucs_utf16(conv_cap)) {
     	if (is_ucs_utf32(conv_cap)) {
@@ -204,86 +207,94 @@ void UNI_latin_oconv (c3)
 	    UCS2PUT2(c2,c1);
 	};
     } else if (is_ucs_utf8(conv_cap)) {
-	if (c3 < 0x80) {
-	    SKFputc(c3); 
-	} else if (c3 < 0x800) {
-	    utf8put2(c3);
+	if (ch < 0x80) {
+	    SKFputc(ch); 
+	} else if (ch < 0x800) {
+	    utf8put2(ch);
 	} else {
-	    utf8put3(c3);
+	    utf8put3(ch);
 	};
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
-    	if (!is_prohibit(c3)) o_p_encode(c3);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
-	    out_undefined(c3,SKF_OUT_PROHIBIT);
-	};
+    	if (!is_prohibit(ch)) o_p_encode(ch);
+	else out_undefined(ch,SKF_OUT_PROHIBIT);
+		/* idn prohibit table (rfch454) is discarded	   */
 #endif
     } else if (is_ucs_utf7(conv_cap)) {
 	if (!is_utf7_shift) {
 	    set_utf7_shift; SKFputc('+');
 	};
-	SKFUTF7ENCODE(c3);
+	SKFUTF7ENCODE(ch);
     };
 }
 
 /* --- CJK kana plane -------------------------------------------- */
-void UNI_cjkkana_oconv (c3)
-    skf_ucode    c3;
+/* 0x3000 - 0x4dff */
+/*@-globstate@*/
+void UNI_cjkkana_oconv (ch)
+    skf_ucode    ch;
 {
-    int	c1,c2;
+    int	c2,c3;
+    unsigned long nc;
 
-    c1 = c3 & 0xff;
-    c2 = (c3 >> 8) & 0xff;
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," uni_cjkkana:%04x",c3);
+    if (is_vv_debug) fprintf(stderr," uni_cjkkana:%04x",ch);
 #endif
-    if (o_encode) out_UNI_encode(c3,c3);
+    if (o_encode) out_UNI_encode(ch,ch);
 
-    if ((c3 == 0x3000) && (!(sup_space_conv))) {
+    if ((ch == 0x3000) && (!(sup_space_conv))) {
 	UNI_ascii_oconv((skf_ucode)(A_SP));
 	if (!(is_spconv_1)) UNI_ascii_oconv((skf_ucode)(A_SP));
 	return;
-    } else if (c3 >= 0x3400) {
-	if (sup_cjk_ext_a) {
-	    out_undefined(c3,SKF_OUNDEF);
+    } else if (ch >= CJK_A_TBL_START) {
+	if (((out_codeset == codeset_nyukan_utf16) ||
+		(out_codeset == codeset_nyukan_utf8)) 
+		&& (uni_o_cjk_a != NULL)) {
+	    nc = uni_o_cjk_a[ch - CJK_A_TBL_START];
+	    if (nc == 0) {
+		out_undefined(ch,SKF_OUNDEF);
+		return;
+	    } else {
+	    	ch = (skf_ucode)nc;
+	    };
+	} else if (sup_cjk_ext_a) {
+	    out_undefined(ch,SKF_OUNDEF);
 	    return;
 	};
-    } else if (use_ms_compat && ((c3 == 0x3099) || (c3 == 0x309a))) {
-	c1 += 2;
+    } else if (use_ms_compat && ((ch == 0x3099) || (ch == 0x309a))) {
+	ch += 2;
     };
     if (is_ucs_utf16(conv_cap)) {
+	c2 = ch & 0xff;
+	c3 = (ch >> 8) & 0xff;
     	if (is_ucs_utf32(conv_cap)) {
-	    UCS4PUT4(0,0,c2,c1);
+	    UCS4PUT4(0,0,c3,c2);
 	} else {
-	    UCS2PUT2(c2,c1);
+	    UCS2PUT2(c3,c2);
 	};
     } else if (is_ucs_utf8(conv_cap)) {
-	utf8put3(c3);
+	utf8put3(ch);
     } else if (is_ucs_utf7(conv_cap)) {
 	if (!is_utf7_shift) {
 	    set_utf7_shift; SKFputc('+');
 	};
-	SKFUTF7ENCODE((c2 << 8) + c1);
+	SKFUTF7ENCODE(ch);
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
-    	if (!is_prohibit(c3)) o_p_encode(c3);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
-	    out_undefined(c3,SKF_OUT_PROHIBIT);
-	};
+    	if (!is_prohibit(ch)) o_p_encode(ch);
+	else out_undefined(ch,SKF_OUT_PROHIBIT);
+		/* idn prohibit table (rfc3454) is discarded	   */
 #endif
     } else;
 }
 
 /* --- CJK plane ------------------------------------------------- */
+/*@-globstate@*/
 void UNI_cjk_oconv(c1)
 skf_ucode c1;
 {
     int	c3,c4;
-
-    c3 = (c1 >> 8) & 0xff;
-    c4 = c1 & 0xff;
+    unsigned long nc;
 
 #ifdef SKFDEBUG
     if (is_vv_debug) fprintf(stderr," uni_cjk:%04x",c1);
@@ -291,12 +302,37 @@ skf_ucode c1;
     if (o_encode) out_UNI_encode(c1,c1);
 
     if (is_ucs_utf16(conv_cap)) {
+	if ((out_codeset == codeset_nyukan_utf16) 
+		&& (uni_o_kanji != NULL)) {
+	    nc = uni_o_kanji[c1 - 0x4e00];
+	    if (nc == 0) {
+		out_undefined(c1,SKF_OUNDEF);
+		return;
+	    } else {
+	    	c1 = (skf_ucode)nc;
+	    };
+	} else;
+
+	c3 = (c1 >> 8) & 0xff;
+	c4 = c1 & 0xff;
+
     	if (is_ucs_utf32(conv_cap)) {
 	    UCS4PUT4(0,0,c3,c4);
 	} else {
 	    UCS2PUT2(c3,c4);
 	};
     } else if (is_ucs_utf8(conv_cap)) {
+	if ((out_codeset == codeset_nyukan_utf8) 
+		&& (uni_o_kanji != NULL)) {
+	    nc = uni_o_kanji[c1 - 0x4e00];
+	    if (nc == 0) {
+		out_undefined(c1,SKF_OUNDEF);
+		return;
+	    } else {
+	    	c1 = (skf_ucode)nc;
+	    };
+	} else;
+
 	utf8put3(c1);
     } else if (is_ucs_utf7(conv_cap)) {
 	if (!is_utf7_shift) {
@@ -306,97 +342,113 @@ skf_ucode c1;
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1)) o_p_encode(c1);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
-	    out_undefined(c1,SKF_OUT_PROHIBIT);
-	};
+	else out_undefined(c1,SKF_OUT_PROHIBIT);
+		/* idn prohibit table (rfc3454) is discarded	   */
 #endif
     };
 }
 
 /* --- ozone and non-BMP ----------------------------------------- */
-void UNI_ozone_oconv(c3)
-skf_ucode c3;
+/*@-globstate@*/
+void UNI_ozone_oconv(ch)
+skf_ucode ch;
 {
-    long c1;
-    int c2,c4;
+    long c3;
+    int c1,c2,c4;
+    unsigned long nc;
 
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," uni_ozone:%04x",c3);
+    if (is_vv_debug) fprintf(stderr," uni_ozone:%06x",ch);
 #endif
-    if (o_encode) out_UNI_encode(c3,c3);
+    if (o_encode) out_UNI_encode(ch,ch);
+
+    if ((out_codeset == codeset_nyukan_utf16) ||
+	(out_codeset == codeset_nyukan_utf8)) {
+	if ((ch >= CJK_B_TBL_START) && (ch <= CJK_B_TBL_END)
+		&& (uni_o_cjk_b != NULL)) {
+	    nc = uni_o_cjk_b[ch - CJK_B_TBL_START];
+	} else if ((ch >= CJK_C_TBL_START) && (ch <= CJK_C_TBL_END)
+		&& (uni_o_cjk_c != NULL)) {
+	    nc = uni_o_cjk_c[ch - CJK_C_TBL_START];
+	} else nc = 0;
+
+	if (nc == 0) {
+	    out_undefined(ch,SKF_OUNDEF);
+	    return;
+	} else ch = (skf_ucode)nc;
+    } else;
 
     if (is_ucs_utf16(conv_cap)) {
     	if (is_ucs_utf32(conv_cap)) {
-	    c1 = c3 & 0xffU;
-	    c2 = (c3 >> 8) & 0xffU;
-	    c3 = (c3 >> 16) & 0xffU;
-	    c4 = (c3 >> 24) & 0xffU;
-	    UCS4PUT4(c4,c3,c2,(int)c1);
+	    c3 = ch & 0xffU;
+	    c2 = (ch >> 8) & 0xffU;
+	    c1 = (ch >> 16) & 0xffU;
+	    c4 = (ch >> 24) & 0xffU;
+	    UCS4PUT4(c4,c1,c2,(int)c3);
 	} else {
-	    if ((c3 <= 0x10ffff) && (c3 >= 0x10000) && !limit_ucs2) {
+	    if ((ch <= UNI_HIGH_LIMIT) && (ch >= 0x10000) && !limit_ucs2) {
 		/* generate surrogate pair			   */
-		c1 = ((c3 >> 10) - 0x040) & 0x3ff;
-		c2 = (int) (((c1 >> 8) & 0x3) + 0xd8);
-		c4 = (int) (c1 & 0xff);
+		c3 = ((ch >> 10) - 0x040) & 0x3ffU;
+		c2 = (int) (((c3 >> 8) & 0x3) + 0xd8);
+		c4 = (int) (c3 & 0xffU);
 		UCS2PUT2(c2,c4);
-		c2 = ((c3 >> 8) & 0x3) + 0xdc;
-		c4 = c3 & 0xff;
+		c2 = ((ch >> 8) & 0x3) + 0xdc;
+		c4 = ch & 0xff;
 		UCS2PUT2(c2,c4);
-	    } else if (c3 >= 0x110000) {
-		out_undefined(c3,SKF_UNSURG);
-	    } else if (limit_ucs2 && (c3 >= 0x10000)) {
-		out_undefined(c3,SKF_UCOMPAT);
+	    } else if (ch > UNI_HIGH_LIMIT) {
+		out_undefined(ch,SKF_UNSURG);
+	    } else if (limit_ucs2 && (ch >= 0x10000)) {
+		out_undefined(ch,SKF_UCOMPAT);
 	    } else {	/* hangul anf Yi */
-		c2 = (int) ((c3 >> 8) & 0xff);
-		c4 = (int) (c3 & 0xff);
+		c2 = (int) ((ch >> 8) & 0xff);
+		c4 = (int) (ch & 0xff);
 		UCS2PUT2(c2,c4);
 	    };
 	};
     } else if (is_ucs_utf7(conv_cap)) {
-	if ((c3 <= 0x10ffff) && (c3 >= 0x10000) && !limit_ucs2) {
+	if ((ch <= UNI_HIGH_LIMIT) && (ch >= 0x10000) && !limit_ucs2) {
 	    /* generate surrogate pair			   */
-	    c1 = ((c3 >> 10) - 0x040) & 0x3ff;
-	    c2 = (int) (((c1 >> 8) & 0x3) + 0xd8);
-	    c4 = (int) (c1 & 0xff) + (c2 << 8);
+	    c3 = ((ch >> 10) - 0x040) & 0x3ff;
+	    c2 = (int) (((c3 >> 8) & 0x3) + 0xd8);
+	    c4 = (int) (c3 & 0xff) + (c2 << 8);
 	    SKFUTF7ENCODE(c4);
-	    c2 = ((c3 >> 8) & 0x3) + 0xdc;
-	    c4 = (c3 & 0xff) + (c2 << 8);
+	    c2 = ((ch >> 8) & 0x3) + 0xdc;
+	    c4 = (ch & 0xff) + (c2 << 8);
 	    SKFUTF7ENCODE(c4);
-	} else if (c3 >= 0x110000) {
-	    out_undefined(c3,SKF_UNSURG);
-	} else if (limit_ucs2 && (c3 >= 0x10000)) {
-	    out_undefined(c3,SKF_UCOMPAT);
+	} else if (ch > UNI_HIGH_LIMIT) {
+	    out_undefined(ch,SKF_UNSURG);
+	} else if (limit_ucs2 && (ch >= 0x10000)) {
+	    out_undefined(ch,SKF_UCOMPAT);
 	} else {	/* hangul anf Yi */
 	    if (!is_utf7_shift) {
 		set_utf7_shift; SKFputc('+');
 	    };
-	    SKFUTF7ENCODE(c3);
+	    SKFUTF7ENCODE(ch);
 	};
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
-    	if (!is_prohibit(c3)) o_p_encode(c3);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
-	    out_undefined(c3,SKF_OUT_PROHIBIT);
-	};
+    	if (!is_prohibit(ch)) o_p_encode(ch);
+	else out_undefined(ch,SKF_OUT_PROHIBIT);
+		/* idn prohibit table (rfc3454) is discarded	   */
 #endif
     } else {
-	if ((c3 <= 0x10ffff) && (c3 >= 0x10000) && !limit_ucs2) {
-	    utf8put4(c3);
-	} else if ((c3 < 0xd800) && (c3 >= 0xa000)) {
-	    utf8put3(c3);
+	if ((ch <= UNI_HIGH_LIMIT) && (ch >= 0x10000) && !limit_ucs2) {
+	    utf8put4(ch);
+	} else if ((ch < 0xd800) && (ch >= 0xa000)) {
+	    utf8put3(ch);
 	} else 
-	    out_undefined(c3,SKF_UCOMPAT);
+	    out_undefined(ch,SKF_UCOMPAT);
     };
 }
 
 
 /* --- private plane --------------------------------------------- */
+/*@-globstate@*/
 void UNI_private_oconv(c1)
 skf_ucode c1;
 {
     int	c3,c4;
+    unsigned long nc;
 
     c3 = (c1 >> 8) & 0xff;
     c4 = c1 & 0xff;
@@ -404,14 +456,24 @@ skf_ucode c1;
 #ifdef SKFDEBUG
     if (is_vv_debug) fprintf(stderr," uni_priv:%04x",c1);
 #endif
+
+    if ((out_codeset == codeset_nyukan_utf16) ||
+	(out_codeset == codeset_nyukan_utf8)) {
+	if ((c1 >= PRV_TBL_START) && (c1 < CPT_TBL_START)
+		&& (uni_o_prv != NULL)) {
+	    nc = uni_o_prv[c1 - PRV_TBL_START];
+	} else nc = c1;
+
+	if (nc == 0) {
+	    out_undefined(c1,SKF_OUNDEF);
+	    return;
+	} else c1 = (skf_ucode)nc;
+    } else;
+
     if (o_encode) out_UNI_encode(c1,c1);
 
     if (c1 < 0xe000) {
-	if (c1 < 0xe880) {
-	    lig_x0213_out(c1);
-	} else {
-	    out_undefined(c1,SKF_IOUTUNI);
-	};
+	lig_x0213_out(c1);
 	return;
     };
     if (is_ucs_utf16(conv_cap)) {
@@ -428,10 +490,8 @@ skf_ucode c1;
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1) || force_private_idn_o) o_p_encode(c1);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
-	    out_undefined(c1,SKF_OUT_PROHIBIT);
-	};
+	else out_undefined(c1,SKF_OUT_PROHIBIT);
+		/* idn prohibit table (rfc3454) is discarded	   */
 #endif
     } else {
 	utf8put3(c1);
@@ -440,61 +500,74 @@ skf_ucode c1;
 
 /* --- compatibility plane --------------------------------------- */
 /* --- U-F900 to U-FFFD ------------------------------------------ */
-void UNI_compat_oconv (c2)
-    skf_ucode    c2;
-{ skf_ucode	c3,c4,c5;
+/*@-globstate@*/
+void UNI_compat_oconv (c1)
+    skf_ucode    c1;
+{ 
+    skf_ucode	c3,c4,c5;
     int		u0,u1;
-    int		c1;
+    int		c2;
+    unsigned long nc;
 
-    c1 = (c2 & 0xff);
-    c3 = (c2 >> 8) & 0xff;
+    if ((out_codeset == codeset_nyukan_utf16) ||
+	(out_codeset == codeset_nyukan_utf8)) {
+	if ((c1 >= CPT_TBL_START) && (c1 <= CPT_TBL_END)
+		&& (uni_o_compat != NULL)) {
+	    nc = uni_o_compat[c1 - CPT_TBL_START];
+	} else nc = c1;
+
+	if (nc == 0) {
+	    out_undefined(c1,SKF_OUNDEF);
+	    return;
+	} else c1 = (skf_ucode)nc;
+    } else;
+
+    c2 = (c1 & 0xff);
+    c3 = (c1 >> 8) & 0xff;
     u0 = (ucode_undef >> 8) & 0xff;
     u1 = ucode_undef & 0xff;
     c4 = u1;
 
 #ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr," ucod_compat:%04x",c2);
+    if (is_vv_debug) fprintf(stderr," ucod_compat:%04x",c1);
 #endif
-    if (o_encode) out_UNI_encode(c2,c2);
+    if (o_encode) out_UNI_encode(c1,c1);
 
-    if ((c3 == 0xfe) && (c1 < 0x10)) { /* variation selector	   */
+    if ((c3 == 0xfe) && (c2 < 0x10)) { /* variation selector	   */
     	if (!enable_var_cntl) return;	/* do nothing		   */
-    } else if (nkf_compat && (c2 == 0xffe5)) {
+    } else if (nkf_compat && (c1 == 0xffe5)) {
     		/* nkf doesn't use fullwidth Yen sign		   */
     	post_oconv(0xa5);
 	return;
-    } else if (c1 >= 0xfffe) {
-    	out_undefined(c1,SKF_OUNDEF);
-	return;
     } else if ((use_compat) && 
-	((!sup_cjk_cmp) || (c2 < 0xfa30) || (c2 > 0xfa6a))) {
-	    c4 = c1;
+	((!sup_cjk_cmp) || (c1 < 0xfa30) || (c1 > 0xfa6a))) {
+	    c4 = c2;
     } else if ((c3 >= 0xf9) && (c3 < 0xfb)) { /* CJK compat kanji  */
-	cjk_compat_parse(c2);
+	cjk_compat_parse(c1);
 	return;		/* suppress output			   */
     } else if (c3 != 0xff) {	/* NOT width variants		   */
 	c3 = u0; 
 /* --- 0xffxx ---- */
-    } else if (c1 < 0x5f) {
-    	c3 = 0; c4 = c1 + 0x20U;
-    } else if ((c1 > 0x61) && (c1 < 0xa0)) {
-	c3 = 0x30; c4 = uni_t_x201[c1-0x61];
-    } else if ((c1 >= 0xa0) && (c1 <= 0xdf)) {
-	if (c1 == 0xa0) { c3 = 0x31; c4 = 0x64; }
-	else if (c1 <= 0xbf) { c3 = 0x31; c4 = c1 - 0x70; }
-	else if ((c1 >= 0xc2) && (c1 <= 0xc7)) { 
-		c3 = 0x31; c4 = c1 - 0x6d; }
-	else if ((c1 >= 0xca) && (c1 <= 0xcf)) { 
-		c3 = 0x31; c4 = c1 - 0x6b; }
-	else if ((c1 >= 0xd2) && (c1 <= 0xd7)) { 
-		c3 = 0x31; c4 = c1 - 0x69; }
-	else if ((c1 >= 0xda) && (c1 <= 0xdf)) { 
-		c3 = 0x31; c4 = c1 - 0x67; }
+    } else if (c2 < 0x5f) {
+    	c3 = 0; c4 = c2 + 0x20U;
+    } else if ((c2 > 0x61) && (c2 < 0xa0)) {
+	c3 = 0x30; c4 = uni_t_x201[c2-0x61];
+    } else if ((c2 >= 0xa0) && (c2 <= 0xdf)) {
+	if (c2 == 0xa0) { c3 = 0x31; c4 = 0x64; }
+	else if (c2 <= 0xbf) { c3 = 0x31; c4 = c2 - 0x70; }
+	else if ((c2 >= 0xc2) && (c2 <= 0xc7)) { 
+		c3 = 0x31; c4 = c2 - 0x6d; }
+	else if ((c2 >= 0xca) && (c2 <= 0xcf)) { 
+		c3 = 0x31; c4 = c2 - 0x6b; }
+	else if ((c2 >= 0xd2) && (c2 <= 0xd7)) { 
+		c3 = 0x31; c4 = c2 - 0x69; }
+	else if ((c2 >= 0xda) && (c2 <= 0xdf)) { 
+		c3 = 0x31; c4 = c2 - 0x67; }
 	else { c3 = u0; };
-    } else if ((c1 >= 0xe0) && (c1 <= 0xef)) {
-	lig_compat(c2);
+    } else if ((c2 >= 0xe0) && (c2 <= 0xef)) {
+	lig_compat(c1);
 	return;
-    } else if (c1 == 0xfd) {
+    } else if (c2 == 0xfd) {
 	c3 = 0xff; c4 = 0xfd;
     } else {
 	c3 = u0; 
@@ -542,11 +615,9 @@ void UNI_compat_oconv (c2)
 	};
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
-    	if (!is_prohibit(c2)) o_p_encode(c2);
-	else { /* idn prohibit table (rfc3454) is discarded	   */
-	    o_p_encode(A_SP);
-	    out_undefined(c2,SKF_OUT_PROHIBIT);
-	};
+    	if (!is_prohibit(c1)) o_p_encode(c1);
+	else out_undefined(c1,SKF_OUT_PROHIBIT);
+		/* idn prohibit table (rfc3454) is discarded	   */
 #endif
     };
 }
@@ -615,32 +686,44 @@ void utf8_finish_procedure()
 }
 
 /* --- output utilities ------------------------------------------ */
-void utf8_urioutstr(x)
+/*@-globstate@*//*@-statictrans@*/
+char *utf8_urioutstr(x)
 skf_ucode x;
 {
-    SKFputc(0x25);
-    SKFputc(hex_conv_table[(x >> 4) & 0x0fU]);
-    SKFputc(hex_conv_table[(x & 0x0fU)]);
+    if (x < 0x80) {
+    	(void)snprintf(ucod_cbuf,1,"%c",x);
+    } else if (x < 0x800) {
+	(void)snprintf(ucod_cbuf,6,"\%02x\%02x",
+		(ZF23VAL + ((x & ZL23MSK) >> 6)),
+		((x & Z1MSK) | ZCVAL));
+    } else if (x < 0x10000) {
+	(void)snprintf(ucod_cbuf,9,"\%02x\%02x\%02x",
+		(ZF3VAL + ((x & ZL3MSK) >> 12)),
+		(ZF2VAL + ((x & ZL2MSK) >> 6)),
+		((x & Z1MSK) | ZCVAL));
+    } else if ((x <= 0x10ffff) && (x >= 0x10000) && !limit_ucs2) {
+	(void)snprintf(ucod_cbuf,12,"\%02lx\%02lx\%02x\%02x",
+		(ZF4VAL + ((x & ZL4MSK) >> 18)),
+		(ZF2VAL + ((x & ZL43MSK) >> 12)),
+		(ZF2VAL + ((x & ZL2MSK) >> 6)),
+		((x & Z1MSK) | ZCVAL));
+    } else ;
+
+    return(ucod_cbuf);
 }
 
 void utf8_uriout(x)
 skf_ucode x;
 {
-    if (x < 0x80) {
-    	utf8_urioutstr(x);
-    } else if (x < 0x800) {
-    	utf8_urioutstr(ZF23VAL + ((x & ZL23MSK) >> 6));
-    	utf8_urioutstr((x & Z1MSK) | ZCVAL);
-    } else if (x < 0x10000) {
-    	utf8_urioutstr(ZF3VAL + ((x & ZL3MSK) >> 12));
-    	utf8_urioutstr(ZF2VAL + ((x & ZL2MSK) >> 6));
-    	utf8_urioutstr((x & Z1MSK) | ZCVAL);
-    } else if ((x <= 0x10ffff) && (x >= 0x10000) && !limit_ucs2) {
-    	utf8_urioutstr(ZF4VAL + ((x & ZL4MSK) >> 18));
-    	utf8_urioutstr(ZF2VAL + ((x & ZL43MSK) >> 12));
-    	utf8_urioutstr(ZF2VAL + ((x & ZL2MSK) >> 6));
-    	utf8_urioutstr(((x & Z1MSK) | ZCVAL));
-    } else ;
+    char *sym;
+
+    if ((sym = utf8_urioutstr(x)) != NULL) {
+    	while (*sym == '\0') {
+	    SKFputc(*sym++);
+	};
+    } else {
+	out_undefined(x,SKF_OUNDEF);
+    };
 }
 
 #ifdef ACE_SUPPORT
@@ -657,14 +740,6 @@ skf_ucode ch;
     else if (((ch >= 0x2000) && (ch <= 0x200f))
 	|| ((ch >= 0x2028) && (ch <= 0x202f))
     	|| (ch == 0x205f) || (ch == 0x3000)) return(1);
-    /* mapped to nothing */
-    else if ((ch == 0xad) || (ch == 0x1806) || (ch == 0x2006)
-	|| (ch == 0x034f) || (ch == 0xfeff) 
-	|| ((ch >= 0x180b) && (ch <= 0x180d))
-	|| ((ch >= 0x200b) && (ch <= 0x200d))
-	|| ((ch >= 0xfe00) && (ch <= 0xfe0f))
-	|| ((ch >= 0x1d173) && (ch <= 0x1d17a))
-	) return(2);
     /* controls */
     else if ((ch == A_DEL) || (ch == 0x06dd) || (ch == 0x070f)
 	|| (ch == 0x180e) || (ch == 0xfeff) 
@@ -782,7 +857,7 @@ skf_ucode cc;		/* table lookup result */
 					/* variation selector	   */
 		    if (enable_var_cntl) 
 			encode_pslenset(&plen,&slen,cc);
-		} else if ((use_compat) && (c3 == 0xfa) &&
+		} else if ((use_compat) && 
 		    ((!sup_cjk_cmp) || (cc < 0xfa30) || (cc > 0xfa6a))) {
 		    encode_pslenset(&plen,&slen,cc);
 		} else if ((c3 >= 0xf9) && (c3 < 0xfb)) {
